@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,6 +10,7 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -17,7 +18,7 @@
 // 
 //
 
-/* $Id: render_handler.h,v 1.17.2.2 2006/11/03 17:35:02 udog Exp $ */
+/* $Id: render_handler.h,v 1.41 2007/06/04 16:59:03 strk Exp $ */
 
 #ifndef RENDER_HANDLER_H
 #define RENDER_HANDLER_H
@@ -114,7 +115,7 @@
 ///
 /// (this applies to the whole Gnash playback architecture)
 ///
-/// After advancing the root movie (see Gui::advance_movie) it is checked
+/// After advancing the root movie (see gnash::Gui::advance_movie) it is checked
 /// which region of the stage has been changed visibly (by computing the 
 /// bounds around updated characters). This has two advantages:
 /// 
@@ -125,25 +126,31 @@
 /// for one frame. This could be implemented in Gnash, too.
 /// 
 /// 2nd, it can detect still frames (like a stopped movie). gui.cpp can
-/// detect these and completely avoids to call any rendering function.
+/// detect these and completely avoid calling any rendering function.
 /// 
 /// Of course, the most critical part is detection of changes. There is a 
-/// method called set_invalidated() which gets called whenever a critical
-/// property of a instance gets updated, like when it changes position, for
-/// example. It's really important to always call set_invalidated() whenever 
-/// code is added that changes the character instance in a visible way.
+/// method gnash::character::set_invalidated() which gets called whenever a
+/// critical property of a instance gets updated, like when it changes
+/// position, for example.
+/// It's really important to *always* call set_invalidated() *before* 
+/// any call that changes the character instance in a visible way.
 /// 
 /// Even if no renderer really uses this information it has effects when
 /// skipping unchanged frames. If necessary, this feature can be switched
 /// off easily in gui.cpp (maybe using a runtime option?).
 ///
-/// Note the updated region is only passed to the GUI, which is itself 
-/// responsible of informing the renderer. This is because it's pointless
+/// Note the updated region is only passed to the gnash::Gui, which is itself 
+/// responsible of informing the renderer (see gnash::Gui::set_invalidated_region).
+/// This is because it's pointless
 /// to have a renderer which updates only a small part of the stage when
-/// the GUI shows it all since the area aroung the region is undefined.
+/// the GUI shows it all since the area around the region is undefined.
 /// However, there can be a GUI which supports update regions without needing
 /// the renderer to do so (for example, to save time during blitting).
 /// The GUI can also completely ignore the region information. 
+///
+/// It's also importanto to note that the bounds passed to the GUI are just
+/// a hint and the GUI /is/ allowed to further process and alter the information
+/// in any way.
 /// 
 /// As for the integer/float discussion: I used rect (floats) because all
 /// the bounds calculation involves floats anyway and so it's probably
@@ -157,7 +164,8 @@
 #include "tu_config.h" // for DSOEXPORT
 
 #include "shape_character_def.h"  
-#include "generic_character.h"    
+#include "generic_character.h"
+#include "Range2d.h"
 
 
 // Forward declarations.
@@ -189,14 +197,15 @@ class render_cache_object
 class DSOEXPORT render_cache_manager
 {
 public:
-  //Virtual dtor, removes compiler warning.
-  virtual ~render_cache_manager(){}
-  /// Clears the cache completely (necessary for runtime shapes / drawing API)
-  virtual void clear() 
-  {
-    // TODO: Make this abstract to force real implementation!!
-    // nop
-  } 
+	//Virtual dtor, removes compiler warning.
+	virtual ~render_cache_manager(){}
+
+	/// Clears the cache completely (necessary for runtime shapes / drawing API)
+	virtual void clear() 
+	{
+		// TODO: Make this abstract to force real implementation!!
+		// nop
+	} 
 };
 
 
@@ -241,20 +250,72 @@ public:
 	/// Delete the given bitmap info class.
 	virtual void	delete_bitmap_info(bitmap_info* bi) = 0;
 
+	/// The different video frame formats
+	enum video_frame_format
+	{
+		NONE,
+		YUV,
+		RGB
+	};
+
+	/// Returns the format the current renderer wants videoframes in.
+	virtual int videoFrameFormat() = 0;
+	
+	/// Draws a video frame. 
+	//
+	/// The frame has already been decoded and is available in the format
+	/// specified by videoFrameFormat().	  
+	///     	  
+	/// @param frame The RGB or YUV video buffer frame.
+	///   Ownership of the buffer is left to the caller.
+	///
+	/// @param mat The matrix with world coordinates used to retrieve the x
+	///   and y coordinate of the video object. The scaling of the matrix only
+	///   refers to the Flash instance, *not* to the video inside that instance.
+	///   When a video object is placed on the stage and the loaded video is
+	///   smaller, then the matrix is still an "identity matrix". However, if
+	///   the video object is scaled via ActionScript, for example, then the
+	///   matrix will change. This means the renderer has to find the correct
+	///   scaling for the video inside the bounds.                             	  
+	///
+	/// @param bounds The minX/minY fields of this rect are always zero. 
+	///   The width and height determine the size of the Flash video instance
+	///   on the stage (in TWIPS) prior to matrix transformations.         
+	///
+	virtual void drawVideoFrame(image::image_base* frame, const matrix* mat, const rect* bounds) = 0;
+
 	/// Sets the update region (called prior to begin_display).
 	//
+	/// The renderer 
+	/// might do clipping and leave the region outside these bounds unchanged,
+	/// but he is allowed to change them if that makes sense. After rendering
+	/// a frame the area outside the invalidated region can be undefined and 
+	/// is not used. 
+	///
 	/// It is not required for all renderers.
-	/// Parameters are world coordinates.
+	/// Parameters are world coordinates (TWIPS).
 	///
 	/// For more info see page \ref region_update.
 	///
-	virtual void set_invalidated_region(const rect /*bounds*/) {    
+	virtual void set_invalidated_region(const rect& /*bounds*/) {    
+		// implementation is optional    
+	}
+
+	virtual void set_invalidated_regions(const InvalidatedRanges& /*ranges*/) {    
 		// implementation is optional    
 	}
 	
-  /// Converts world coordinates to pixel coordinates
-  virtual void world_to_pixel(int *x, int *y, const float world_x, 
-    const float world_y) = 0;  
+	/// Converts world coordinates to pixel coordinates
+	virtual geometry::Range2d<int> world_to_pixel(const rect& worldbounds) = 0;
+  
+	virtual geometry::Range2d<int> world_to_pixel(const geometry::Range2d<float>& worldbounds)
+	{
+  		if ((worldbounds.isNull() || worldbounds.isWorld())) 	
+  			return worldbounds;
+
+		return world_to_pixel(rect(worldbounds.getMinX(), worldbounds.getMinY(),
+		                           worldbounds.getMaxX(), worldbounds.getMaxY()));  
+	}
 		
 	/// Bracket the displaying of a frame from a movie.
 	//
@@ -279,21 +340,21 @@ public:
 	//
 	/// Can be used to draw empty boxes and cursors.
 	virtual void	draw_line_strip(const void* coords, int vertex_count,
-    const rgba color) = 0;
+			const rgba color) = 0;
     
-  /// Draw a simple, solid filled polygon (no outline).
-  //
-  /// This can't be used for 
-  /// Flash shapes but is intended for internal drawings like bounding boxes 
-  /// (editable text fields) and similar. The polygon should not contain 
-  /// self-intersections. If you do not wish a outline or a fill, then simply 
-  /// set the alpha value to zero.
-  ///
-  /// The polygon need NOT be closed (ie: this function will automatically
-  /// add an additional vertex to close it.
-  ///
-  virtual void  draw_poly(const point* corners, size_t corner_count, 
-    const rgba fill, const rgba outline) = 0;
+	/// Draw a simple, solid filled polygon (no outline).
+	//
+	/// This can't be used for 
+	/// Flash shapes but is intended for internal drawings like bounding boxes 
+	/// (editable text fields) and similar. The polygon should not contain 
+	/// self-intersections. If you do not wish a outline or a fill, then simply 
+	/// set the alpha value to zero.
+	///
+	/// The polygon need NOT be closed (ie: this function will automatically
+	/// add an additional vertex to close it.
+	///
+	virtual void  draw_poly(const point* corners, size_t corner_count, 
+		const rgba fill, const rgba outline) = 0;
     
 		
 	/// Set line and fill styles for mesh & line_strip rendering.
@@ -331,97 +392,241 @@ public:
 	/// character instance.
 	///
 	virtual void draw_shape_character(shape_character_def *def, 
-    character *inst) {
-    
-    // TODO: I don't like that there is a draw_shape_character() version with
-    // arbitrary fill and line styles as this may break caching...
+		character *inst)
+	{
+
+		// check if the character needs to be rendered at all
+		rect cur_bounds;
+
+		cur_bounds.expand_to_transformed_rect(inst->get_world_matrix(), 
+				def->get_bound());
+				
+		if (!bounds_in_clipping_area(cur_bounds))
+		{
+			  return; // no need to draw
+		}
+			
+
+		// TODO: I don't like that there is a draw_shape_character() version with
+		// arbitrary fill and line styles as this may break caching...
+
+		// render the character
+		draw_shape_character(def, 
+		inst->get_world_matrix(), 
+		inst->get_world_cxform(),
+		inst->get_parent()->get_pixel_scale(),
+		def->get_fill_styles(),
+		def->get_line_styles());
+
+	}
   
-    draw_shape_character(def, 
-      inst->get_world_matrix(), 
-      inst->get_world_cxform(),
-      inst->get_parent()->get_pixel_scale(),
-      def->get_fill_styles(),
-      def->get_line_styles());
+	/// \brief
+	/// Checks if the given bounds are (partially) in the current drawing clipping
+	/// area.
+	//
+	/// A render handler implementing invalidated bounds should implement
+	/// this method to avoid rendering of characters that are not visible anyway.
+	/// By default this method always returns true, which will ensure correct
+	/// rendering. If possible, it should be re-implemented by the renderer 
+	/// handler for better performance.
+	/// 'bounds' contains TWIPS coordinates.
+	///
+	/// TODO: Take a Range2d<T> rather then a gnash::rect ?
+	///       Would T==int be good ? TWIPS as integer types ?
+	///
+	/// See also gnash::renderer::bounds_in_clipping_area
+	///
+	virtual bool bounds_in_clipping_area(const rect& bounds) {
+		return bounds_in_clipping_area(bounds.getRange());
+	}
+  
+	virtual bool bounds_in_clipping_area(const InvalidatedRanges& ranges)
+	{
+		for (unsigned int rno=0; rno<ranges.size(); rno++) 
+		{
+			if (bounds_in_clipping_area(ranges.getRange(rno)))
+				return true;
+		}
+  			
+		return false;
+	}
+  
+	virtual bool bounds_in_clipping_area(const geometry::Range2d<float>& /*bounds*/) {
+		return true;
+	}
 
-  }
-
-  /// \brief
-  /// Draws the given character definition with the given transformations and
-  /// styles. 
+	/// \brief
+	/// Draws the given character definition with the given transformations and
+	/// styles. 
 	virtual void draw_shape_character(shape_character_def *def, 
-    const matrix& mat,
-    const cxform& cx,
-    float pixel_scale,
-    const std::vector<fill_style>& fill_styles,
-    const std::vector<line_style>& line_styles) = 0;
+		const matrix& mat,
+		const cxform& cx,
+		float pixel_scale,
+		const std::vector<fill_style>& fill_styles,
+		const std::vector<line_style>& line_styles) = 0;
     
-  /// \brief
-  /// Draws a glyph (font character).
-  //
-  /// Glyphs are defined just like shape characters with the difference that
-  /// they do not have any fill or line styles.
-  /// Instead, the shape must be drawn using the given color (solid fill). 
-  /// 
-  /// @param def
-  ///
-  /// @param mat
-  ///
-  /// @param color
-  ///
-  /// @param pixel_scale
-  ///
-  virtual void draw_glyph(shape_character_def *def,
-    const matrix& mat,
-    rgba color,
-    float pixel_scale) = 0;
+	/// \brief
+	/// Draws a glyph (font character).
+	//
+	/// Glyphs are defined just like shape characters with the difference that
+	/// they do not have any fill or line styles.
+	/// Instead, the shape must be drawn using the given color (solid fill). 
+	/// 
+	/// @param def
+	///
+	/// @param mat
+	///
+	/// @param color
+	///
+	/// @param pixel_scale
+	///
+	virtual void draw_glyph(shape_character_def *def, const matrix& mat,
+		rgba color, float pixel_scale) = 0;
     
-  /// The render handler can choose if it wishes to use textured glyphs 
-  /// (pre-computed bitmaps which are used for small text sizes) or if 
-  /// draw_glyph() should be used in any case. When glyph textures are not
-  /// desired, then draw_bitmap() is never called in the *current* version.  
-  virtual bool allow_glyph_textures() = 0;
+	/// The render handler can choose if it wishes to use textured glyphs 
+	/// (pre-computed bitmaps which are used for small text sizes) or if 
+	/// draw_glyph() should be used in any case. When glyph textures are not
+	/// desired, then draw_bitmap() is never called in the *current* version.  
+	virtual bool allow_glyph_textures() = 0;
     
     
   /// This function returns the color at any position in the stage. It is used
   /// for automatic testing only, it should not be used for anything else!
-  /// world_x and world_y are world coordinates (twips) and the color of the
-  /// nearest pixel is returned.
-  virtual void get_pixel(rgba& /*color_return*/, float /*world_x*/, 
-    float /*world_y*/) {
-    
-    log_msg("get_pixel() not implemented for this renderer");
-    assert(0);    
-    
-  }
+  /// x and y are pixel coordinates (<0 won't make any sense) and the color of 
+  /// the nearest pixel is returned.
+  /// The function returns false when the coordinates are outside the 
+  /// main frame buffer.
+  virtual bool getPixel(rgba& /*color_return*/, int /*x*/, int /*y*/)
+  {
 
-  /// Sets the x/y scale for the movie  
-  virtual void set_scale(float /*xscale*/, float /*yscale*/) {
-    // nop
-  }    
+    log_msg("getPixel() not implemented for this renderer");
+    assert(0);    
+    return false; // avoid compiler warning    
+  }
+  
+  
+  /// Returns the average RGB color for a square block on the stage. The 
+  /// width and height of the block is defined by "radius" and x/y refer
+  /// to the center of the block. radius==1 equals getPixel() and radius==0
+  /// is illegal. For even "radius" values, the center point is not exactly
+  /// defined. 
+  /// The function returns false when at least one pixel of the block was
+  /// outside the main frame buffer. In that case the value in color_return
+  /// is undefined.
+  /// This implementation is provided for simplicity. Renderers should
+  /// implement a specialized version for better performance.
+  virtual bool getAveragePixel(rgba& color_return, int x, int y, 
+    unsigned int radius)
+  {
+  
+    assert(radius>0); 
+  
+    // optimization:
+    if (radius==1)
+      return getPixel(color_return, x, y);
+  
+    unsigned int r=0, g=0, b=0, a=0;
+    
+    x -= radius/2;
+    y -= radius/2;
+    
+    int xe = x+radius;
+    int ye = y+radius;
+
+    rgba pixel;
+    
+    for (int yp=y; yp<ye; yp++)
+    for (int xp=x; xp<xe; xp++)
+    {
+      if (!getPixel(pixel, xp, yp))
+        return false;
+        
+      r += pixel.m_r;      
+      g += pixel.m_g;      
+      b += pixel.m_b;      
+      a += pixel.m_a;      
+    }
+    
+    int pcount = radius*radius; 
+    color_return.m_r = r / pcount; 
+    color_return.m_g = g / pcount; 
+    color_return.m_b = b / pcount; 
+    color_return.m_a = a / pcount; 
+    
+    return true;
+  }
+  
+  
+	/// \brief
+	/// Initializes the renderer for off-screen rendering used by the  
+	/// testsuite.
+	///
+	/// This is a special function used for testcases ONLY. It is used by
+	/// MovieTester to prepare the renderer for off-screen rendering 
+	/// without any GUI. The renderer is responsible to do all required
+	/// steps so that rendering is possible after the call. This may mean
+	/// that the renderer allocates memory for the given stage size.
+	/// 
+	/// The function returns false when the renderer is not able to do
+	/// off-screen rendering (default).
+	///
+	/// Note the function may be called again afterwards, resizing the stage.
+	/// Any number of calls to this function is possible and the renderer
+	/// is responsible to resize any buffer instead of wasting memory. 
+	///
+	/// @param width stage width in pixels
+	///
+	/// @param height stage height in pixels
+	virtual bool initTestBuffer(unsigned /*width*/, unsigned /*height*/)
+	{
+		return false;
+	}
+
+	/// Return color depth (bits per pixel) or 0 if unknown/unimplemented.
+	//
+	/// Default implementation returns 0 (unknown).
+	///
+	/// TODO: this should be a pure abstract function, just don't want
+	///       to scan ogl and cairo backend for an implementation *now*
+	///       but would be needed for automated testing... Quinn, can you help ?
+	///
+	virtual unsigned int getBitsPerPixel() const
+	{
+		return 0;
+	}
+  
+	/// Sets the x/y scale for the movie  
+	virtual void set_scale(float /*xscale*/, float /*yscale*/) {
+		// nop
+	}
+
+	virtual void get_scale(point& /*scale*/) {
+	}
     
 protected:
 
-  // Cached fill style list with just one entry used for font rendering
-  std::vector<fill_style>	m_single_fill_styles;
-  
-  // Dummy line styles list without entries (do not add anything!!)
-  std::vector<line_style>	m_dummy_line_styles;
-  
-  // Dummy, neutral color transformation (do not change!!)
-  cxform m_neutral_cxform;
-  
-  // Sets m_single_fill_styles to one solid fill with the given color 
-  void need_single_fill_style(const rgba& color){
-  
-    if (m_single_fill_styles.size() == 0) {  
-      fill_style dummy;
-    
-      m_single_fill_styles.push_back(dummy);
-    }
-    
-    m_single_fill_styles[0].set_color(color);
-  
-  } //need_single_fill_style
+	/// Cached fill style list with just one entry used for font rendering
+	std::vector<fill_style>	m_single_fill_styles;
+
+	/// Dummy line styles list without entries (do not add anything!!)
+	std::vector<line_style>	m_dummy_line_styles;
+
+	/// Dummy, neutral color transformation (do not change!!)
+	cxform m_neutral_cxform;
+
+	/// Sets m_single_fill_styles to one solid fill with the given color 
+	void need_single_fill_style(const rgba& color)
+	{
+
+		if (m_single_fill_styles.size() == 0)
+		{	
+			fill_style dummy;
+			m_single_fill_styles.push_back(dummy);
+		}
+
+		m_single_fill_styles[0].set_color(color);
+
+	} 
 
 }; // class render_handler
 
