@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,10 +10,14 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+// 
+//
+
 #ifndef _PLAYER_H_
 #define _PLAYER_H_
 
@@ -23,7 +27,8 @@
 
 #include "tu_config.h"
 
-#include "gnash.h"
+#include "gnash.h" // still needed ?
+#include "sound_handler.h" // for visibility of sound_handler destructor
 #include "gui.h"
 
 #include <string>
@@ -32,7 +37,7 @@
 // Forward declarations
 namespace gnash
 {
-	class movie_interface;
+	class sprite_instance;
 }
 
 
@@ -68,7 +73,9 @@ public:
 	///
 	int run(int argc, char* argv[],
 		const char* infile, const char* url=NULL);
-
+#if 0
+        int run(const char* infile);
+#endif
 	float setScale(float s);
 
 	// milliseconds per frame
@@ -123,9 +130,9 @@ private:
 
 	void init_gui();
 
-	static void setFlashVars(movie_interface& m, const std::string& varstr);
+	static void setFlashVars(sprite_instance& m, const std::string& varstr);
 
-	static void fs_callback(movie_interface* movie,
+	static void fs_callback(sprite_instance* movie,
 			const char* command, const char* args);
 
 	// Movie parameters (for -P)
