@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,11 +10,16 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: kde_glue.h,v 1.5.2.2 2006/10/31 19:31:49 rsavoye Exp $ */
+//
+//
+
+
+/* $Id: kde_glue.h,v 1.11 2007/05/28 15:41:00 ann Exp $ */
 
 #include "gnash.h"
 
@@ -26,7 +31,7 @@ namespace gnash
 class KdeGlue
 {
   public:
-    virtual ~KdeGlue() { delete _drawing_area; }
+    virtual ~KdeGlue() { };
     virtual bool init(int argc, char **argv[]) = 0;
 
     virtual void prepDrawingArea(QWidget *drawing_area) = 0;
