@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,6 +10,7 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -17,7 +18,7 @@
 // 
 //
 
-/* $Id: curl_adapter.h,v 1.3.2.1 2006/10/30 14:28:47 rsavoye Exp $ */
+/* $Id: curl_adapter.h,v 1.7 2007/05/28 15:41:01 ann Exp $ */
 
 #ifndef CURL_ADAPTER_H
 #define CURL_ADAPTER_H
@@ -42,6 +43,20 @@ namespace curl_adapter
 ///
 DSOEXPORT tu_file* make_stream(const char* url);
 
+/// \brief
+/// Returns a read-only tu_file stream that fetches data
+/// from an url getting posted to.
+//
+/// The caller owns the returned tu_file*.  
+///
+/// @param url
+///	The url to post to.
+///
+/// @param postdata
+///	The url-encoded post data
+///
+DSOEXPORT tu_file* make_stream(const char* url, const std::string& postdata);
+
 }
 
 #endif // CURL_ADAPTER_H
