@@ -1,21 +1,20 @@
-// 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
-// 
+// network.cpp:  TCP/IP support, for Gnash.
+//
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+//
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-// 
-//
 //
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +35,7 @@
 # include <windows.h>
 # include <sys/stat.h>
 # include <io.h>
+# include <ws2tcpip.h>
 #else
 # include <sys/time.h>
 # include <unistd.h>
@@ -54,10 +54,6 @@
 
 using namespace std;
 
-namespace {
-gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
-}
-
 namespace gnash {
 
 static const char *DEFAULTPROTO = "tcp";
@@ -67,24 +63,32 @@ static const short DEFAULTPORT  = RTMP;
 #define INADDR_NONE  0xffffffff
 #endif
 
-Network::Network() : _ipaddr(INADDR_ANY), _sockfd(0), _listenfd(0), _port(0), _connected(false), _debug(false), _timeout(5)
+Network::Network()
+	:
+	_ipaddr(INADDR_ANY),
+	_sockfd(0),
+	_listenfd(0),
+	_port(0),
+	_connected(false),
+	_debug(false),
+	_timeout(5)
 {
-    //log_msg("%s: \n", __PRETTY_FUNCTION__);
+    //GNASH_REPORT_FUNCTION;
 #ifdef HAVE_WINSOCK_H
     WORD wVersionRequested;
     WSADATA wsaData;
     wVersionRequested = MAKEWORD(1, 1);		// Windows Sockets 1.1
     if (WSAStartup( wVersionRequested, &wsaData ) != 0) {
-        printf("ERROR: could not find a usable WinSock DLL.\n");
+        log_error(_("Could not find a usable WinSock DLL"));
         exit(1);
     }
 #endif
 
 }
 
-Network::~Network() 
+Network::~Network()
 {
-    //log_msg("%s: \n", __PRETTY_FUNCTION__);
+    //GNASH_REPORT_FUNCTION;
 #ifdef HAVE_WINSOCK_H
     WSACleanup();
 #else
@@ -92,17 +96,6 @@ Network::~Network()
 #endif
 }
 
-Network &
-Network::operator = (Network &net)
-{
-    _sockfd = net.getFileFd();
-    _port = net.getPort();
-    _host = net.getHost();
-    _connected = net.connected();
-    _timeout = net.getTimeout();
-    return *this;
-}
-
 // Description: Create a tcp/ip network server. This creates a server
 //              that listens for incoming socket connections. This
 //              support IP aliasing on the host, and will sequntially
@@ -110,20 +103,28 @@ Network::operator = (Network &net)
 bool
 Network::createServer(void)
 {
-    log_msg("%s: \n", __PRETTY_FUNCTION__);
+//    GNASH_REPORT_FUNCTION;
+
     return createServer(DEFAULTPORT);
 }
 
+// FIXME: Should also support IPv6 (AF_INET6)
 bool
 Network::createServer(short port)
 {
-    log_msg("%s: \n", __PRETTY_FUNCTION__);
+//    GNASH_REPORT_FUNCTION;
+
     struct protoent *ppe;
     struct sockaddr_in sock_in;
     int             on, type;
     int             retries = 0;
     in_addr_t       nodeaddr;
 
+    if (port < 1024) {
+	log_error(_("Can't connect to privileged port #%d"), port);
+	return false;
+    }
+
     const struct hostent *host = gethostbyname("localhost");
     struct in_addr *thisaddr = reinterpret_cast<struct in_addr *>(host->h_addr_list[0]);
     _ipaddr = thisaddr->s_addr;
@@ -136,75 +137,68 @@ Network::createServer(short port)
     // Accept incoming connections on any IP number
     sock_in.sin_addr.s_addr = INADDR_ANY;
 #endif
-    
+
     _ipaddr = sock_in.sin_addr.s_addr;
     sock_in.sin_family = AF_INET;
     sock_in.sin_port = htons(port);
-  
+
     if ((ppe = getprotobyname(DEFAULTPROTO)) == 0) {
-        // error, wasn't able to get a protocol entry
-        log_msg("WARNING: unable to get protocol entry for %s\n",
+        log_error(_("unable to get protocol entry for %s"),
                 DEFAULTPROTO);
         return false;
     }
-    
+
     // set protocol type
     if (DEFAULTPROTO == "udp") {
         type = SOCK_DGRAM;
     } else {
         type = SOCK_STREAM;
     }
-    
+
     // Get a file descriptor for this socket connection
     _listenfd = socket(PF_INET, type, ppe->p_proto);
-    
+
     // error, wasn't able to create a socket
     if (_listenfd < 0) {
-        log_msg("unable to create socket: %s\n", strerror(errno));
+        log_error(_("unable to create socket: %s"), strerror(errno));
         return true;
     }
-    
+
     on = 1;
     if (setsockopt(_listenfd, SOL_SOCKET, SO_REUSEADDR,
                    (char *)&on, sizeof(on)) < 0) {
-        log_msg("setsockopt SO_REUSEADDR failed!\n");
+        log_error(_("setsockopt SO_REUSEADDR failed"));
         return false;
     }
-    
+
     retries = 0;
-    
+
     nodeaddr = inet_lnaof(*thisaddr);
     while (retries < 5) {
         if (bind(_listenfd, reinterpret_cast<struct sockaddr *>(&sock_in),
                  sizeof(sock_in)) == -1) {
-            log_msg("WARNING: unable to bind to port %hd! %s\n",
+            log_error(_("unable to bind to port %hd: %s"),
                     port, strerror(errno));
 //                    inet_ntoa(sock_in.sin_addr), strerror(errno));
             retries++;
         }
 
-#if 0
-        char                ascip[32];
-        inet_ntop(AF_INET, &_ipaddr, ascip, INET_ADDRSTRLEN);
-        log_msg("Host Name is %s, IP is %s", host->h_name, ascip);
-#endif
-    
-        log_msg("Server bound to service on port: %hd, %s using fd #%d\n",
-                ntohs(sock_in.sin_port), inet_ntoa(sock_in.sin_addr),
-                _listenfd);
-        
+	if (_debug) {
+//		char  ascip[INET_ADDRSTRLEN];
+//		inet_ntop(sock_in.sin_family, &_ipaddr, ascip, INET_ADDRSTRLEN);
+		char *ascip = ::inet_ntoa(sock_in.sin_addr);
+		log_msg(_("Server bound to service on %s, port %hd, using fd %d"),
+		    ascip, ntohs(sock_in.sin_port),
+		    _listenfd);
+	}
+
         if (type == SOCK_STREAM && listen(_listenfd, 5) < 0) {
-            log_msg("ERROR: unable to listen on port: %hd: %s ",
+            log_error(_("unable to listen on port: %hd: %s "),
                 port, strerror(errno));
             return false;
         }
 
         _port = port;
-    
-#if 0
-        log_msg("Listening for net traffic on fd #%d\n", _sockfd);
-#endif
-    
         return true;
     }
     return false;
@@ -216,7 +210,7 @@ Network::createServer(short port)
 bool
 Network::newConnection(void)
 {
-    log_msg("%s: \n", __PRETTY_FUNCTION__);
+//    GNASH_REPORT_FUNCTION;
 
     return newConnection(true);
 }
@@ -224,24 +218,25 @@ Network::newConnection(void)
 bool
 Network::newConnection(bool block)
 {
-    log_msg("%s: \n", __PRETTY_FUNCTION__);
+//    GNASH_REPORT_FUNCTION;
+
     struct sockaddr	newfsin;
     socklen_t		alen;
     int			ret;
     struct timeval        tval;
     fd_set                fdset;
     int                   retries = 3;
-  
+
     alen = sizeof(struct sockaddr_in);
-  
+
 #ifdef NET_DEBUG
-    log_msg("Trying to accept net traffic on fd #%d\n", _sockfd);
+    log_msg(_("Trying to accept net traffic on fd %d"), _sockfd);
 #endif
-  
+
     if (_listenfd <= 2) {
         return false;
     }
-  
+
     while (retries--) {
         // We use select to wait for the read file descriptor to be
         // active, which means there is a client waiting to connect.
@@ -251,37 +246,37 @@ Network::newConnection(bool block)
 //             FD_SET(fileno(stdin), &fdset);
 //         }
         FD_SET(_listenfd, &fdset);
-    
+
         // Reset the timeout value, since select modifies it on return. To
         // block, set the timeout to zero.
         tval.tv_sec = 1;
         tval.tv_usec = 0;
-    
+
         if (block) {
             ret = select(_listenfd+1, &fdset, NULL, NULL, NULL);
         } else {
             ret = select(_listenfd+1, &fdset, NULL, NULL, &tval);
         }
-    
+
         if (FD_ISSET(0, &fdset)) {
-            log_msg("There is data at the console for stdin!");
+            log_msg(_("There is data at the console for stdin"));
             return true;
         }
 
         // If interupted by a system call, try again
         if (ret == -1 && errno == EINTR) {
-            log_msg("The accept() socket for fd #%d was interupted by a system call!\n", _listenfd);
+            log_msg(_("The accept() socket for fd %d was interupted by a system call"), _listenfd);
         }
-    
+
         if (ret == -1) {
-            log_msg("ERROR: The accept() socket for fd #%d never was available for writing!",
+            log_msg(_("The accept() socket for fd %d never was available for writing"),
                     _listenfd);
             return false;
         }
-    
+
         if (ret == 0) {
             if (_debug) {
-                log_msg("ERROR: The accept() socket for fd #%d timed out waiting to write!\n",
+                log_msg(_("The accept() socket for fd %d timed out waiting to write"),
                         _listenfd);
             }
         }
@@ -291,13 +286,15 @@ Network::newConnection(bool block)
     fcntl(_listenfd, F_SETFL, O_NONBLOCK); // Don't let accept() block
 #endif
     _sockfd = accept(_listenfd, &newfsin, &alen);
-  
+
     if (_sockfd < 0) {
-        log_msg("unable to accept : %s\n", strerror(errno));
+        log_error(_("unable to accept: %s"), strerror(errno));
         return false;
     }
-  
-    log_msg("Accepting tcp/ip connection on fd #%d\n", _sockfd);
+
+    if (_debug) {
+	log_msg(_("Accepting tcp/ip connection on fd %d"), _sockfd);
+    }
 
     return true;
 }
@@ -306,23 +303,31 @@ Network::newConnection(bool block)
 bool
 Network::createClient(void)
 {
+    GNASH_REPORT_FUNCTION;
+
     return createClient("localhost", RTMP);
 }
 bool
 Network::createClient(short /* port */)
 {
+    GNASH_REPORT_FUNCTION;
+
     return false;
 }
 
 bool
 Network::createClient(const char *hostname)
 {
-    return createClient(hostname, RTMP);        
+    GNASH_REPORT_FUNCTION;
+
+    return createClient(hostname, RTMP);
 }
-    
+
 bool
 Network::createClient(const char *hostname, short port)
 {
+    GNASH_REPORT_FUNCTION;
+
     struct sockaddr_in  sock_in;
     fd_set              fdset;
     struct timeval      tval;
@@ -331,23 +336,25 @@ Network::createClient(const char *hostname, short port)
     char                thishostname[MAXHOSTNAMELEN];
     struct protoent     *proto;
 
+    assert( ! connected() );
+
     if (port < 1024) {
-        log_error("Can't connect to priviledged port #%hd!\n", port);
+        log_error(_("Can't connect to privileged port %hd"), port);
         _connected = false;
         return false;
     }
 
-    log_msg("%s: to host %s at port %d\n", __FUNCTION__, hostname, port);
-  
+    log_msg(_("%s: to host %s at port %d"), __FUNCTION__, hostname, port);
+
     memset(&sock_in, 0, sizeof(struct sockaddr_in));
     memset(&thishostname, 0, MAXHOSTNAMELEN);
     if (strlen(hostname) == 0) {
         if (gethostname(thishostname, MAXHOSTNAMELEN) == 0) {
-            log_msg("The hostname for this machine is %s.\n", thishostname);
+            log_msg(_("The hostname for this machine is %s"), thishostname);
         } else {
-            log_msg("Couldn't get the hostname for this machine!\n");
+            log_msg(_("Couldn't get the hostname for this machine"));
             return false;
-        }   
+        }
     }
     const struct hostent *hent = ::gethostbyname(hostname);
     if (hent > 0) {
@@ -357,9 +364,9 @@ Network::createClient(const char *hostname, short port)
     sock_in.sin_port = ntohs(static_cast<short>(port));
 
 #if 0
-    char ascip[32];
-    inet_ntop(AF_INET, &sock_in.sin_addr.s_addr, ascip, INET_ADDRSTRLEN);
-    log_msg("The IP address for this client socket is %s\n", ascip);
+    char ascip[INET_ADDRSTRLEN];
+    inet_ntop(sock_in.sin_family, &sock_in.sin_addr.s_addr, ascip, INET_ADDRSTRLEN);
+    log_msg(_("The IP address for this client socket is %s"), ascip);
 #endif
 
     proto = ::getprotobyname("TCP");
@@ -367,7 +374,7 @@ Network::createClient(const char *hostname, short port)
     _sockfd = ::socket(PF_INET, SOCK_STREAM, proto->p_proto);
     if (_sockfd < 0)
         {
-            log_error("unable to create socket : %s\n", strerror(errno));
+            log_error(_("unable to create socket: %s"), strerror(errno));
             _sockfd = -1;
             return false;
         }
@@ -378,36 +385,36 @@ Network::createClient(const char *hostname, short port)
         // active, which means there is a client waiting to connect.
         FD_ZERO(&fdset);
         FD_SET(_sockfd, &fdset);
-    
+
         // Reset the timeout value, since select modifies it on return. To
         // block, set the timeout to zero.
         tval.tv_sec = 5;
         tval.tv_usec = 0;
-    
+
         ret = ::select(_sockfd+1, &fdset, NULL, NULL, &tval);
 
         // If interupted by a system call, try again
         if (ret == -1 && errno == EINTR)
             {
-                log_msg("The connect() socket for fd #%d was interupted by a system call!\n",
+                log_msg(_("The connect() socket for fd %d was interupted by a system call"),
                         _sockfd);
                 continue;
             }
-    
+
         if (ret == -1)
             {
-                log_msg("The connect() socket for fd #%d never was available for writing!\n",
+                log_msg(_("The connect() socket for fd %d never was available for writing"),
                         _sockfd);
 #ifdef HAVE_WINSOCK_H
                 ::shutdown(_sockfd, 0); // FIXME: was SHUT_BOTH
 #else
                 ::shutdown(_sockfd, SHUT_RDWR);
 #endif
-                _sockfd = -1;      
+                _sockfd = -1;
                 return false;
             }
         if (ret == 0) {
-            log_error("The connect() socket for fd #%d timed out waiting to write!\n",
+            log_error(_("The connect() socket for fd %d timed out waiting to write"),
                       _sockfd);
             continue;
         }
@@ -415,15 +422,20 @@ Network::createClient(const char *hostname, short port)
         if (ret > 0) {
             ret = ::connect(_sockfd, reinterpret_cast<struct sockaddr *>(&sock_in), sizeof(sock_in));
             if (ret == 0) {
-                log_msg("\tport %d at IP %s for fd #%d\n", port,
-                        ::inet_ntoa(sock_in.sin_addr), _sockfd);
+		char *ascip = ::inet_ntoa(sock_in.sin_addr);
+// 		char ascip[INET_ADDRSTRLEN];
+// 		inet_ntop(sock_in.sin_family, &sock_in.sin_addr.s_addr, ascip, INET_ADDRSTRLEN);
+                log_msg(_("\tport %d at IP %s for fd %d"), port,
+                        ascip, _sockfd);
                 _connected = true;
+                assert(_sockfd > 0);
                 return true;
             }
             if (ret == -1) {
-                log_msg("The connect() socket for fd #%d never was available for writing!\n",
+                log_error(_("The connect() socket for fd %d never was available for writing"),
                         _sockfd);
-                _sockfd = -1;      
+                _sockfd = -1;
+                assert(!_connected);
                 return false;
             }
         }
@@ -431,35 +443,39 @@ Network::createClient(const char *hostname, short port)
     //  ::close(_sockfd);
     //  return false;
 
-    printf("\tConnected at port %d on IP %s for fd #%d\n", port,
+    printf("\tConnected at port %d on IP %s for fd #%d", port,
            ::inet_ntoa(sock_in.sin_addr), _sockfd);
-  
+
 #ifndef HAVE_WINSOCK_H
     fcntl(_sockfd, F_SETFL, O_NONBLOCK);
 #endif
 
     _connected = true;
+    assert(_sockfd > 0);
     return true;
 }
 
 bool
 Network::closeNet()
-{  
-//    log_msg("%s: \n", __PRETTY_FUNCTION__);
+{
+//    GNASH_REPORT_FUNCTION;
+
     if (_sockfd > 0) {
         closeNet(_sockfd);
         _sockfd = 0;
-        _connected = false;        
+        _connected = false;
     }
-  
+
     return false;
 }
 
 bool
 Network::closeNet(int sockfd)
 {
+//    GNASH_REPORT_FUNCTION;
+
     int retries = 0;
-    
+
     // If we can't close the socket, other processes must be
     // locked on it, so we wait a second, and try again. After a
     // few tries, we give up, cause there must be something
@@ -468,14 +484,14 @@ Network::closeNet(int sockfd)
     if (sockfd <= 0) {
         return true;
     }
-  
+
     while (retries < 3) {
         if (sockfd) {
             // Shutdown the socket connection
 #if 0
             if (shutdown(sockfd, SHUT_RDWR) < 0) {
                 if (errno != ENOTCONN) {
-                    cerr << "WARNING: Unable to shutdown socket for fd #\n"
+                    cerr << "WARNING: Unable to shutdown socket for fd #"
                          << sockfd << strerror(errno) << endl;
                 } else {
                     cerr << "The socket using fd #" << sockfd
@@ -483,43 +499,46 @@ Network::closeNet(int sockfd)
                     return true;
                 }
             }
-#endif 
+#endif
             if (close(sockfd) < 0) {
-                log_msg("WARNING: Unable to close the socket for fd%d\n%s\n",
+                log_error(_("Unable to close the socket for fd %d: %s"),
                         sockfd, strerror(errno));
 #ifndef HAVE_WINSOCK_H
                 sleep(1);
 #endif
                 retries++;
             } else {
-                log_msg("Closed the socket on fd #%d\n", sockfd);
+                log_msg(_("Closed the socket on fd %d"), sockfd);
                 return true;
             }
         }
-    } 
+    }
     return false;
 }
 // Description: Close an open socket connection.
 bool
 Network::closeConnection(void)
 {
-    log_msg("%s: \n", __FUNCTION__);
+    GNASH_REPORT_FUNCTION;
 
     closeConnection(_sockfd);
+    _sockfd = 0;
     _listenfd = 0;
     _connected = false;
-  
+
     return false;
 }
 
 bool
 Network::closeConnection(int fd)
 {
-    log_msg("%s: \n", __FUNCTION__);
+    GNASH_REPORT_FUNCTION;
+
     if (fd > 0) {
-        closeConnection(fd);
+        ::close(fd);
+//        closeNet(fd);
     }
-  
+
     return false;
 }
 
@@ -561,45 +580,42 @@ Network::readNet(int fd, char *buffer, int nbytes, int timeout)
         FD_ZERO(&fdset);
         FD_SET(fd, &fdset);
 
-        // Reset the timeout value, since select modifies it on return
-        // Reset the timeout value, since select modifies it on return
-        if (timeout <= 0) {
-            timeout = 5;
-        }
-        tval.tv_sec = timeout;
-        tval.tv_usec = 0;
-        ret = select(fd+1, &fdset, NULL, NULL, &tval);
-        
+        if (timeout < 0) {
+	    tval.tv_sec = 5;
+	    tval.tv_usec = 0;
+	    ret = select(fd+1, &fdset, NULL, NULL, &tval);
+        } else {
+	    ret = select(fd+1, &fdset, NULL, NULL, NULL);
+	}
+
         // If interupted by a system call, try again
         if (ret == -1 && errno == EINTR) {
-            dbglogfile << "The socket for fd #" << fd
-                       << " we interupted by a system call!" << endl;
+            log_error (_("The socket for fd %d was interupted by a system call"), fd);
         }
-        
+
         if (ret == -1) {
-            dbglogfile << "The socket for fd #" << fd
-                       << " never was available for reading!" << endl;
+            log_error (_("The socket for fd %d was never available for reading"), fd);
             return -1;
         }
-        
+
         if (ret == 0) {
-            dbglogfile << "The socket for fd #" << fd
-                << " timed out waiting to read!" << endl;
+            log_error (_("The socket for fd %d timed out waiting to read"), fd);
             return -1;
         }
-    
+
         ret = read(fd, buffer, nbytes);
-        dbglogfile << "read " << ret << " bytes from fd #"
-                   << fd << endl;
+	if (_debug) {
+	    log_msg (_("read %d bytes from fd %d"), ret, fd);
+	}
     }
-    
+
     return ret;
 
 }
 
 // Write to the connection
 int
-Network::writeNet(std::string buffer)
+Network::writeNet(const std::string& buffer)
 {
     return writeNet(buffer.c_str(), buffer.size());
 }
@@ -653,7 +669,7 @@ Network::writeNet(int fd, char const *buffer, int nbytes, int timeout)
     if (fd) {
         FD_ZERO(&fdset);
         FD_SET(fd, &fdset);
-        
+
         // Reset the timeout value, since select modifies it on return
         if (timeout <= 0) {
             timeout = 5;
@@ -661,52 +677,47 @@ Network::writeNet(int fd, char const *buffer, int nbytes, int timeout)
         tval.tv_sec = timeout;
         tval.tv_usec = 0;
         ret = select(fd+1, NULL, &fdset, NULL, &tval);
-        
+
         // If interupted by a system call, try again
         if (ret == -1 && errno == EINTR) {
-            dbglogfile << "The socket for fd #" << fd
-                << " we interupted by a system call!" << endl;
+            log_error (_("The socket for fd %d was interupted by a system call"), fd);
         }
-        
+
         if (ret == -1) {
-            dbglogfile << "The socket for fd #" << fd
-                << " never was available for writing!" << endl;
+            log_error (_("The socket for fd %d was never available for writing"), fd);
         }
-        
+
         if (ret == 0) {
-            dbglogfile << "The socket for fd #" << fd
-                << " timed out waiting to write!" << endl;
+            log_error (_("The socket for fd %d timed out waiting to write"), fd);
         }
-        
+
         ret = write(fd, bufptr, nbytes);
 
         if (ret == 0) {
-            dbglogfile
-                << "Couldn't write any bytes to fd #: " << fd
-                << strerror(errno) << endl;
+            log_error (_("Wrote zero out of %d bytes to fd %d: %s"), 
+		nbytes, fd, strerror(errno));
             return ret;
         }
         if (ret < 0) {
-            dbglogfile << "Couldn't write " << nbytes
-                       << " bytes to fd #" << fd << endl;
-
+            log_error (_("Couldn't write %d bytes to fd %d: %s"), 
+		nbytes, fd, strerror(errno));
             return ret;
         }
         if (ret > 0) {
             bufptr += ret;
             if (ret != nbytes) {
-                dbglogfile << "wrote " << ret << " bytes to fd #" << fd
-                           << " expected " << nbytes << endl;
+		if (_debug) {
+		    log_msg (_("wrote %d bytes to fd %d, expected %d"),
+			       ret, fd, nbytes);
+		}
 //                retries++;
             } else {
-                dbglogfile << "wrote " << ret << " bytes to fd #"
-                           << fd << endl;
+		if (_debug) {
+		    log_msg (_("wrote %d bytes to fd %d"),
+			       ret, fd);
+		}
                 return ret;
             }
-            
-            if (ret == 0) {
-                dbglogfile << "Wrote 0 bytes to fd #" << fd << endl;
-            }
         }
     }
 
@@ -718,12 +729,11 @@ Network::writeNet(int fd, char const *buffer, int nbytes, int timeout)
         if ((endtime.tv_sec - starttime.tv_sec) &&
             endtime.tv_usec - starttime.tv_usec)
         {
-            dbglogfile << "took " << endtime.tv_usec - starttime.tv_usec
-                       << " usec to write (" << bytes_written
-                       << " bytes)\n" << endl;
+            log_msg (_("took %d usec to write (%d bytes)"),
+		endtime.tv_usec - starttime.tv_usec, bytes_written);
         }
     }
-#endif    
+#endif
 
     return ret;
 }
@@ -735,7 +745,7 @@ Network::toggleDebug(bool val)
     _debug = val;
 
     // Turn on debugging for the utility methods
-		// Vitaly: recursive on all control paths,
+		// recursive on all control paths,
 		// function will cause runtime stack overflow
 
 		// toggleDebug(true);
@@ -743,3 +753,8 @@ Network::toggleDebug(bool val)
 
 
 } // end of gnash namespace
+
+// Local Variables:
+// mode: C++
+// indent-tabs-mode: t
+// End:
