@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,32 +10,22 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-
-/* $Id: noseek_fd_adapter.cpp,v 1.6.2.2 2006/10/31 10:03:39 strk Exp $ */
-
-#if defined(_WIN32) || defined(WIN32)
-#define snprintf _snprintf
-#endif
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#ifdef HAVE_STDIN_CURL_H
-# define HAVE_LIBCURL 1
-#endif
-
 #include "noseek_fd_adapter.h"
 #include "tu_file.h"
 #include "utility.h"
 #include "GnashException.h"
 #include "log.h"
 #include <unistd.h>
+#include <cstring>
 
 //#define GNASH_NOSEEK_FD_VERBOSE 1
 
@@ -97,6 +87,9 @@ public:
 	/// Return true if EOF has been reached
 	bool eof();
 
+	/// Return zero if everything is fine (?)
+	int err() { return 0; }
+
 	/// Report global position within the file
 	size_t tell();
 
@@ -225,8 +218,8 @@ NoSeekFile::fill_cache(size_t size)
 
 
 	char* buf = new char[bytes_needed];
-	ssize_t bytes_read = read(_fd, (void*)buf, bytes_needed);
-	if ( bytes_read == static_cast<ssize_t>(-1) )
+	int bytes_read = read(_fd, (void*)buf, bytes_needed);
+	if ( bytes_read < 0 )
 	{
 		fprintf(stderr,
 			"Error reading " SIZET_FMT " bytes from input stream",
@@ -251,7 +244,7 @@ NoSeekFile::fill_cache(size_t size)
 		}
 	}
 
-	cache(buf, bytes_read);
+	cache(buf, static_cast<size_t>(bytes_read));
 
 	delete [] buf;
 }
@@ -423,6 +416,14 @@ eof(void* appdata)
 	return stream->eof();
 }
 
+static int
+err(void* appdata)
+{
+	NoSeekFile* stream = (NoSeekFile*) appdata;
+	if ( ! stream ) return TU_FILE_OPEN_ERROR;
+	return stream->err();
+}
+
 static int
 write(const void* /*src*/, int /*bytes*/, void* /*appdata*/)
 {
@@ -489,6 +490,8 @@ make_stream(int fd, const char* cachefilename)
 		seek_to_end, // seek_to_end
 		tell, // tell
 		eof, // get eof
+		err, // get error
+		NULL, // get stream size
 		close);
 }
 
