@@ -12,12 +12,27 @@
 #include "membuf.h"
 #include "log.h"
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 //
 // tu_file functions using FILE
 //
 
 using namespace gnash;
 
+namespace gnash {
+
+static int std_read_func(void* dst, int bytes, void* appdata);
+static int std_write_func(const void* src, int bytes, void* appdata);
+static int std_seek_func(int pos, void *appdata);
+static int std_seek_to_end_func(void *appdata);
+static int std_tell_func(void *appdata);
+static bool std_get_eof_func(void *appdata);
+static int std_get_err_func(void *appdata);
+static long std_get_stream_size_func(void *appdata);
+
 static int
 std_read_func(void* dst, int bytes, void* appdata) 
 // Return the number of bytes actually read.  EOF or an error would
@@ -41,15 +56,26 @@ std_write_func(const void* src, int bytes, void* appdata)
 
 static int
 std_seek_func(int pos, void *appdata)
-// Return 0 on success, or TU_FILE_SEEK_ERROR on failure.
 {
     assert(appdata);
-    clearerr((FILE*) appdata);	// make sure EOF flag is cleared.
-    int	result = fseek((FILE*)appdata, pos, SEEK_SET);
+
+    // TODO: optimize this by caching total stream size ?
+    if (pos > std_get_stream_size_func(appdata))
+    {
+	    return TU_FILE_SEEK_ERROR;
+    }
+
+    FILE* file = static_cast<FILE*>(appdata);
+
+    clearerr(file); // make sure EOF flag is cleared.
+    int	result = fseek(file, pos, SEEK_SET);
     if (result == EOF) {
 	// @@ TODO should set m_error to something relevant based on errno.
 	return TU_FILE_SEEK_ERROR;
     }
+
+    assert ( ftell(file) == pos );
+
     return 0;
 }
 
@@ -71,7 +97,14 @@ std_tell_func(void *appdata)
 // Return the file position, or -1 on failure.
 {
     assert(appdata);
-    return ftell((FILE*)appdata);
+    FILE* f = static_cast<FILE*>(appdata);
+
+    //if ( feof(f) )
+    //assert ( ! feof(f) ); // I guess it's legal to call tell() while at eof.
+
+    int ret = ftell(f);
+    assert(ret <= std_get_stream_size_func(appdata));
+    return ret;
 }
 
 static bool
@@ -86,6 +119,32 @@ std_get_eof_func(void *appdata)
     }
 }
 
+static int
+std_get_err_func(void *appdata)
+// Return true if we're at EOF.
+{
+    if ( ! appdata ) return TU_FILE_OPEN_ERROR;
+    return (ferror((FILE*) appdata));
+}
+
+static long
+std_get_stream_size_func(void *appdata)
+// Return -1 on failure, or the size
+{
+    assert(appdata);
+
+    FILE* f = static_cast<FILE*>(appdata);
+
+    struct stat statbuf;
+    if ( -1 == fstat(fileno(f), &statbuf) )
+    {
+	    log_error("Could not fstat file");
+	    return 0;
+    }
+    return statbuf.st_size;
+}
+
+
 static int
 std_close_func(void *appdata)
 // Return 0 on success, or TU_FILE_CLOSE_ERROR on failure.
@@ -112,10 +171,9 @@ public:
     int		m_position;
     
     filebuf()
-	:
-	m_position(0),
-	m_read_only(false)
 	{
+		m_position = 0; 
+		m_read_only = false;
 	}
     
     filebuf(int size, void* data)
@@ -271,6 +329,26 @@ mem_get_eof_func(void* appdata)
     return buf->m_position >= buf->m_.size();
 }
 
+static int
+mem_get_err_func(void* appdata)
+// Return true if we're positioned at the end of the buffer.
+{
+    filebuf* buf = (filebuf*) appdata;
+    return buf->is_valid();
+}
+
+static long
+mem_get_stream_size(void* appdata)
+// Return the file size, or -1 on failure.
+{
+    assert(appdata);
+    
+    filebuf* buf = (filebuf*) appdata;
+    assert(buf->is_valid());
+    
+    return buf->m_.size();
+}
+
 static int mem_close_func(void* appdata)
 // Return 0 on success, or TU_FILE_CLOSE_ERROR on failure.
 {
@@ -283,7 +361,7 @@ static int mem_close_func(void* appdata)
     
     return 0;
 }
-
+}
 
 //
 // generic functionality
@@ -292,7 +370,7 @@ static int mem_close_func(void* appdata)
 // Create a file using the custom callbacks.
 tu_file::tu_file(void * appdata, read_func rf, write_func wf,
 		 seek_func sf, seek_to_end_func ef, tell_func tf,
-		 get_eof_func gef, close_func cf)
+		 get_eof_func gef, get_err_func ger, get_stream_size_func gss, close_func cf)
 {
     m_data = appdata;
     m_read = rf;
@@ -301,14 +379,15 @@ tu_file::tu_file(void * appdata, read_func rf, write_func wf,
     m_seek_to_end = ef;
     m_tell = tf;
     m_get_eof = gef;
+    m_get_err = ger;
+    m_get_stream_size = gss;
     m_close = cf;
-    m_error = TU_FILE_NO_ERROR;
 }
 
 // Create a file from a standard file pointer.
 tu_file::tu_file(FILE* fp, bool autoclose=false)
 {
-    GNASH_REPORT_FUNCTION;
+    //GNASH_REPORT_FUNCTION;
 
     m_data = (void *)fp;
     m_read = std_read_func;
@@ -317,36 +396,27 @@ tu_file::tu_file(FILE* fp, bool autoclose=false)
     m_seek_to_end = std_seek_to_end_func;
     m_tell = std_tell_func;
     m_get_eof = std_get_eof_func;
+    m_get_err = std_get_err_func;
+    m_get_stream_size = std_get_stream_size_func;
     m_close = autoclose ? std_close_func : NULL;
-    m_error = TU_FILE_NO_ERROR;
 }
 
 // Create a file from the given name and the given mode.
 tu_file::tu_file(const char * name, const char * mode)
 {
-    GNASH_REPORT_RETURN;
+	GNASH_REPORT_RETURN;
 
-    m_data = fopen(name, mode);
+	m_data = fopen(name, mode);
     
-    if (m_data) {
 	m_read = std_read_func;
 	m_write = std_write_func;
 	m_seek = std_seek_func;
 	m_seek_to_end = std_seek_to_end_func;
 	m_tell = std_tell_func;
 	m_get_eof = std_get_eof_func;
+	m_get_err = std_get_err_func;
+	m_get_stream_size = std_get_stream_size_func;
 	m_close = std_close_func;
-	m_error = TU_FILE_NO_ERROR;
-    } else {
-	m_read = NULL;
-	m_write = NULL;
-	m_seek = NULL;
-	m_seek_to_end = NULL;
-	m_tell = NULL;
-	m_get_eof = NULL;
-	m_close = NULL;
-	m_error = TU_FILE_OPEN_ERROR;
-    }
 }
 
 tu_file::tu_file(memory_buffer_enum /* m */)
@@ -360,8 +430,9 @@ tu_file::tu_file(memory_buffer_enum /* m */)
     m_seek_to_end = mem_seek_to_end_func;
     m_tell = mem_tell_func;
     m_get_eof = mem_get_eof_func;
+    m_get_err = mem_get_err_func;
+    m_get_stream_size = mem_get_stream_size;
     m_close = mem_close_func;
-    m_error = TU_FILE_NO_ERROR;
 }
 
 
@@ -376,8 +447,9 @@ tu_file::tu_file(memory_buffer_enum /* m */, int size, void* data)
     m_seek_to_end = mem_seek_to_end_func;
     m_tell = mem_tell_func;
     m_get_eof = mem_get_eof_func;
+    m_get_err = mem_get_err_func;
+    m_get_stream_size = mem_get_stream_size;
     m_close = mem_close_func;
-    m_error = TU_FILE_NO_ERROR;
 }
 
 tu_file::~tu_file()
@@ -390,7 +462,7 @@ void
 tu_file::close() 
 // Close this file.
 { 
-    if (m_close) {
+    if (m_close && m_data) {
 	m_close(m_data);
     }
     m_data = NULL; 
