@@ -10,20 +10,31 @@
 #include "zlib_adapter.h"
 #include "tu_file.h"
 #include "utility.h"
+#include "log.h"
+
+#include <memory>
 
 
-#if !TU_CONFIG_LINK_TO_ZLIB
+#ifndef HAVE_ZLIB_H
 
 
 // Stubs, in case client doesn't want to link to zlib.
 namespace zlib_adapter
 {
-	tu_file*	make_inflater(tu_file* in) { return NULL; }
-	tu_file*	make_deflater(tu_file* out) { return NULL; }
+	std::auto_ptr<tu_file> make_inflater(std::auto_ptr<tu_file> /*in*/)
+	{
+		assert(0); // callers should check this themselves
+		return std::auto_ptr<tu_file>(NULL);
+	}
+	tu_file* make_deflater(tu_file* /*out*/)
+	{
+		assert(0); // callers should check this themselves
+		return NULL;
+	}
 }
 
 
-#else // TU_CONFIG_LINK_TO_ZLIB
+#else // HAVE_ZLIB_H
 
 
 #include <zlib.h>
@@ -36,7 +47,7 @@ namespace zlib_adapter
 	class inflater_impl
 	{
 	private:
-		tu_file*	m_in;
+		std::auto_ptr<tu_file>	m_in;
 		int		m_initial_stream_pos;	// position of the input stream where we started inflating.
 		unsigned char	m_rawdata[ZBUF_SIZE];
 
@@ -50,16 +61,16 @@ namespace zlib_adapter
 		bool		m_at_eof;
 		int		m_error;
 		
-		inflater_impl(tu_file* in)
+		inflater_impl(std::auto_ptr<tu_file> in)
 		// Constructor.
 			:
 			m_in(in),
-			m_initial_stream_pos(in->get_position()),
+			m_initial_stream_pos(m_in->get_position()),
 			m_logical_stream_pos(m_initial_stream_pos),
 			m_at_eof(false),
 			m_error(0)
 		{
-			assert(m_in);
+			assert(m_in.get());
 
 			m_zstream.zalloc = (alloc_func)0;
 			m_zstream.zfree = (free_func)0;
@@ -73,7 +84,7 @@ namespace zlib_adapter
 
 			int	err = inflateInit(&m_zstream);
 			if (err != Z_OK) {
-				//log_error("error: inflater_impl::ctor() inflateInit() returned %d\n", err);
+				gnash::log_error("inflater_impl::ctor() inflateInit() returned %d\n", err);
 				m_error = 1;
 				return;
 			}
@@ -90,6 +101,7 @@ namespace zlib_adapter
 			m_at_eof = 0;
 			int	err = inflateReset(&m_zstream);
 			if (err != Z_OK) {
+				gnash::log_error("inflater_impl::reset() inflateReset() returned %d\n", err);
 				m_error = 1;
 				return;
 			}
@@ -141,9 +153,17 @@ namespace zlib_adapter
 					m_at_eof = true;
 					break;
 				}
+				if (err == Z_BUF_ERROR)
+				{
+					//gnash::log_error("inflater_impl::inflate_from_stream() inflate() returned Z_BUF_ERROR");
+					// we should call inflate again... giving more input or output space !
+					//m_error = 1;
+					break;
+				}
 				if (err != Z_OK)
 				{
 					// something's wrong.
+					gnash::log_error("inflater_impl::inflate_from_stream() inflate() returned %d", err);
 					m_error = 1;
 					break;
 				}
@@ -202,12 +222,14 @@ namespace zlib_adapter
 
 
 	int inflate_seek(int pos, void* appdata)
-	// Try to go to pos.  Return actual pos.
+	// Try to go to pos.  Return 0 on success or TU_FILE_SEEK_ERROR on error.
 	{
 		inflater_impl*	inf = (inflater_impl*) appdata;
 		if (inf->m_error)
 		{
-			return inf->m_logical_stream_pos;
+			gnash::log_debug("Inflater is in error condition");
+			return TU_FILE_SEEK_ERROR;
+			//return inf->m_logical_stream_pos;
 		}
 
 		// If we're seeking backwards, then restart from the beginning.
@@ -230,13 +252,16 @@ namespace zlib_adapter
 			if (bytes_read == 0)
 			{
 				// Trouble; can't seek any further.
+				gnash::log_debug("Trouble: can't seek any further.. ");
+				return TU_FILE_SEEK_ERROR;
 				break;
 			}
 		}
 
-		assert(inf->m_logical_stream_pos <= pos);
+		//assert(inf->m_logical_stream_pos <= pos);
+		assert(inf->m_logical_stream_pos == pos);
 
-		return inf->m_logical_stream_pos;
+		return 0; // inf->m_logical_stream_pos;
 	}
 
 
@@ -280,6 +305,12 @@ namespace zlib_adapter
 		return inf->m_at_eof;
 	}
 
+	inline int inflate_get_err(void* appdata)
+	{
+		inflater_impl*	inf = (inflater_impl*) appdata;
+		return inf->m_error;
+	}
+
 	inline int inflate_close(void* appdata)
 	{
 		inflater_impl*	inf = (inflater_impl*) appdata;
@@ -298,20 +329,24 @@ namespace zlib_adapter
 	}
 
 
-	tu_file*	make_inflater(tu_file* in)
+	std::auto_ptr<tu_file> make_inflater(std::auto_ptr<tu_file> in)
 	{
-		assert(in);
+		assert(in.get());
 
 		inflater_impl*	inflater = new inflater_impl(in);
-		return new tu_file(
-			inflater,
-			inflate_read,
-			inflate_write,
-			inflate_seek,
-			inflate_seek_to_end,
-			inflate_tell,
-			inflate_get_eof,
-			inflate_close);
+		return std::auto_ptr<tu_file> (
+			new tu_file(
+				inflater,
+				inflate_read,
+				inflate_write,
+				inflate_seek,
+				inflate_seek_to_end,
+				inflate_tell,
+				inflate_get_eof,
+				inflate_get_err,
+				NULL, // get stream size
+				inflate_close)
+			);
 	}
 
 
@@ -319,7 +354,7 @@ namespace zlib_adapter
 	// tu_file*	make_deflater(tu_file* out) { ... }
 }
 
-#endif // TU_CONFIG_LINK_TO_ZLIB
+#endif // HAVE_ZLIB_H
 
 
 // Local Variables:
