@@ -15,17 +15,18 @@
 
 namespace cull {
 
-	struct result_info {
+	class result_info {
+	public:
 		bool	culled;	// true when the volume is not visible
-		Uint8	active_planes;	// one bit per frustum plane
+		uint8_t	active_planes;	// one bit per frustum plane
 		
-		result_info(bool c = false, Uint8 a = 0x3f) : culled(c), active_planes(a) {}
+		result_info(bool c = false, uint8_t a = 0x3f) : culled(c), active_planes(a) {}
 	};
 	
 	
 	result_info	compute_box_visibility(const vec3& center, const vec3& extent,
 					       const plane_info frustum[6], result_info in);
-};
+}
 
 
 #endif // CULL_H
