@@ -3,7 +3,7 @@
 // This source code has been donated to the Public Domain.  Do
 // whatever you want with it.
 
-// "view_state", a struct that holds various important rendering
+// "view_state", a class that holds various important rendering
 // states during render traversal.
 
 
@@ -15,10 +15,11 @@
 #include "cull.h"
 
 
-struct view_state
+class view_state
 // Description of basic rendering state.  Passed as an arg to
 // visual::render().
 {
+public:
 	int	m_frame_number;
 	matrix	m_matrix;
 	plane_info	m_frustum[6];	// In local coordinates.
