@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,11 +10,12 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: plugin.h,v 1.27.2.2 2006/10/31 19:33:49 rsavoye Exp $ */
+/* $Id: plugin.h,v 1.32 2007/05/28 15:41:02 ann Exp $ */
 
 #ifndef __PLUGIN_H__
 #define __PLUGIN_H__
@@ -24,13 +25,13 @@
 #endif
 
 #ifndef HAVE_FUNCTION
-	#ifndef HAVE_func
-		#define dummystr(x) # x
-		#define dummyestr(x) dummystr(x)
-		#define __FUNCTION__ __FILE__":"dummyestr(__LINE__)
-	#else
-		#define __FUNCTION__ __func__	
-	#endif
+# ifndef HAVE_func
+#  define dummystr(x) # x
+#  define dummyestr(x) dummystr(x)
+#  define __FUNCTION__ __FILE__":"dummyestr(__LINE__)
+# else
+#  define __FUNCTION__ __func__	
+# endif
 #endif
 
 #ifndef HAVE_PRETTY_FUNCTION
@@ -79,11 +80,11 @@ public:
     NPError SetWindow(NPWindow *aWindow);
 
     NPError NewStream(NPMIMEType type, NPStream *stream, NPBool seekable,
-                      uint16 *stype);
+                      uint16_t *stype);
     NPError DestroyStream(NPStream * stream, NPError reason);
 
-    int32 WriteReady(NPStream *stream);
-    int32 Write(NPStream *stream, int32 offset, int32 len, void *buffer);
+    int32_t WriteReady(NPStream *stream);
+    int32_t Write(NPStream *stream, int32_t offset, int32_t len, void *buffer);
 
     NPError WriteStatus(char *msg) const;
     NPError WriteStatus(std::string msg) const;
