@@ -1,5 +1,6 @@
+// array.cpp:  ActionScript array class, for Gnash.
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,40 +11,37 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+//
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-// 
-//
 //
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-// This needs to be included first for NetBSD systems or we get a weird
-// problem with pthread_t being defined too many times if we use any
-// STL containers.
-#ifdef HAVE_PTHREADS
-#include <pthread.h>
-#endif
-
-#include <string>
-#include <algorithm>
-#include <memory> // for auto_ptr
-
+#include "as_value.h"
 #include "array.h"
-//#include "action.h"
 #include "log.h"
 #include "builtin_function.h" // for Array class
 #include "as_function.h" // for sort user-defined comparator
 #include "fn_call.h"
-#include "as_value.h"
+#include "GnashException.h"
+
+#include <string>
+#include <algorithm>
+#include <memory> // for auto_ptr
+#include <boost/algorithm/string/case_conv.hpp>
+
+//#define GNASH_DEBUG 
+
 
 namespace gnash {
 
 static as_object* getArrayInterface();
+static void attachArrayProperties(as_object& proto);
+static void attachArrayInterface(as_object& proto);
 
 // Default as_value strict weak comparator (string based)
 class AsValueLessThen
@@ -51,7 +49,7 @@ class AsValueLessThen
 public:
 	bool operator() (const as_value& a, const as_value& b)
 	{
-		return ( a.to_tu_string() < b.to_tu_string() );
+		return a.to_string().compare(b.to_string()) < 0;
 	}
 };
 
@@ -61,7 +59,7 @@ class AsValueLessThenDesc
 public:
 	bool operator() (const as_value& a, const as_value& b)
 	{
-		return ( a.to_string() > b.to_string() );
+		return a.to_string().compare(b.to_string()) > 0;
 	}
 };
 
@@ -71,7 +69,12 @@ class AsValueLessThenNoCase
 public:
 	bool operator() (const as_value& a, const as_value& b)
 	{
-		return ( a.to_tu_stringi() < b.to_tu_stringi() );
+		using namespace boost::algorithm;
+
+		std::string strA = to_upper_copy(a.to_string());
+		std::string strB = to_upper_copy(b.to_string());
+
+		return strA.compare(strB) < 0;
 	}
 };
 
@@ -81,7 +84,12 @@ class AsValueLessThenDescNoCase
 public:
 	bool operator() (const as_value& a, const as_value& b)
 	{
-		return ( a.to_tu_stringi() > b.to_tu_stringi() );
+		using namespace boost::algorithm;
+
+		std::string strA = to_upper_copy(a.to_string());
+		std::string strB = to_upper_copy(b.to_string());
+
+		return strA.compare(strB) > 0;
 	}
 };
 
@@ -127,8 +135,8 @@ public:
 		env.push(b);
 
 		as_value ret(false); // bool value
-		fn_call fn(&ret, NULL, &env, 2, 0);
-		_comp(fn);
+		fn_call fn(NULL, &env, 2, 0);
+		ret = _comp(fn);
 		return ( ret.to_bool() );
 	}
 };
@@ -140,7 +148,10 @@ as_array_object::as_array_object()
 	as_object(getArrayInterface()), // pass Array inheritance
 	elements(0)
 {
-    //log_action("%s : %p\n", __FUNCTION__, (void*)this);
+	//IF_VERBOSE_ACTION (
+	//log_action("%s: %p", __FUNCTION__, (void*)this);
+	//)
+	attachArrayProperties(*this);
 }
 
 as_array_object::as_array_object(const as_array_object& other)
@@ -148,14 +159,16 @@ as_array_object::as_array_object(const as_array_object& other)
 	as_object(other),
 	elements(other.elements)
 {
-    //log_action("%s : %p\n", __FUNCTION__, (void*)this);
+    //IF_VERBOSE_ACTION (
+    //log_action("%s: %p", __FUNCTION__, (void*)this);
+    //)
 }
 
 int
-as_array_object::index_requested(const tu_stringi& name)
+as_array_object::index_requested(const std::string& name)
 {
 	as_value temp;
-	temp.set_string(name.c_str());
+	temp.set_string(name);
 	double value = temp.to_number();
 
 	// if we were sent a string that can't convert like "asdf", it returns as NaN. -1 means invalid index
@@ -170,13 +183,13 @@ as_array_object::index_requested(const tu_stringi& name)
 }
 
 void
-as_array_object::push(as_value& val)
+as_array_object::push(const as_value& val)
 {
 	elements.push_back(val);
 }
 
 void
-as_array_object::unshift(as_value& val)
+as_array_object::unshift(const as_value& val)
 {
 	elements.push_front(val);
 }
@@ -187,7 +200,7 @@ as_array_object::pop()
 	// If the array is empty, report an error and return undefined!
 	if (elements.size() <= 0)
 	{
-	    log_warning("tried to pop element from back of empty array, returning undef!\n");
+	    log_error(_("tried to pop element from back of empty array, returning undef"));
 		return as_value(); // undefined
 	}
 
@@ -203,7 +216,7 @@ as_array_object::shift()
 	// If the array is empty, report an error and return undefined!
 	if (elements.size() <= 0)
 	{
-		log_warning("tried to shift element from front of empty array, returning undef!\n");
+		log_error(_("tried to shift element from front of empty array, returning undef"));
 		return as_value(); // undefined
 	}
 
@@ -221,7 +234,7 @@ as_array_object::reverse()
 }
 
 std::string
-as_array_object::join(const std::string& separator)
+as_array_object::join(const std::string& separator, as_environment* env) const
 {
 	// TODO - confirm this is the right format!
 	// Reportedly, flash version 7 on linux, and Flash 8 on IE look like
@@ -240,12 +253,12 @@ as_array_object::join(const std::string& separator)
 			itEnd=elements.end();
 
 		// print first element w/out separator prefix
-		temp += (*it++).to_string();
+		temp += (*it++).to_string(env);
 
 		// print subsequent elements with separator prefix
 		while ( it != itEnd )
 		{
-			temp += separator + (*it++).to_string();
+			temp += separator + (*it++).to_string(env);
 		}
 	}
 
@@ -263,9 +276,9 @@ as_array_object::concat(const as_array_object& other)
 }
 
 std::string
-as_array_object::toString()
+as_array_object::toString(as_environment* env) const
 {
-	return join(",");
+	return join(",", env);
 }
 
 unsigned int
@@ -290,8 +303,18 @@ as_array_object::at(unsigned int index)
 std::auto_ptr<as_array_object>
 as_array_object::slice(unsigned int start, unsigned int one_past_end)
 {
+	assert(one_past_end >= start);
+	assert(one_past_end <= size());
+	assert(start <= size());
+
 	std::auto_ptr<as_array_object> newarray(new as_array_object);
-	newarray->elements.resize(one_past_end - start - 1);
+
+#ifdef GNASH_DEBUG
+	log_msg(_("Array.slice(%u, %u) called"), start, one_past_end);
+#endif
+
+	size_t newsize = one_past_end - start;
+	newarray->elements.resize(newsize);
 
 	// maybe there's a standard algorithm for this ?
 	for (unsigned int i=start; i<one_past_end; ++i)
@@ -303,16 +326,52 @@ as_array_object::slice(unsigned int start, unsigned int one_past_end)
 
 }
 
-/* virtual public, overriding as_object::set_member */
-bool
-as_array_object::get_member(const tu_stringi& name, as_value *val)
+std::auto_ptr<as_array_object>
+as_array_object::splice(unsigned start, unsigned len,
+		const std::vector<as_value>& replace)
 {
-	if ( name == "length" ) 
+	assert(len <= size()-start);
+	assert(start <= size());
+
+#ifdef GNASH_DEBUG
+	std::stringstream ss;
+	ss << "Array.splice(" << start << ", " << len << ", ";
+	std::ostream_iterator<as_value> ostrIter(ss, "," ) ;
+	std::copy(replace.begin(), replace.end(), ostrIter);
+        ss << ") called";
+	log_msg("%s", ss.str().c_str());
+	log_msg(_("Current array is %s"), toString().c_str());
+#endif
+
+	container::iterator itStart = elements.begin()+start;
+	container::iterator itEnd = itStart+len;
+
+	// This will be returned...
+	std::auto_ptr<as_array_object> ret(new as_array_object);
+	
+	// If something has to be removed do it and assign
+	// to the returned object
+	if ( itStart != itEnd )
 	{
-		val->set_double((double)size());
-		return true;
+		ret->elements.assign(itStart, itEnd);
+
+		elements.erase(itStart, itEnd);
 	}
 
+	// Now insert the new stuff, if needed
+	if ( replace.size() )
+	{
+		container::iterator itStart = elements.begin()+start;
+		elements.insert(itStart, replace.begin(), replace.end());
+	}
+
+	return ret;
+}
+
+/* virtual public, overriding as_object::get_member */
+bool
+as_array_object::get_member(const std::string& name, as_value *val)
+{
 	// an index has been requested
 	int index = index_requested(name);
 	if ( index >= 0 && (unsigned int)index < elements.size() )
@@ -324,17 +383,17 @@ as_array_object::get_member(const tu_stringi& name, as_value *val)
 	return get_member_default(name, val);
 }
 
+void
+as_array_object::resize(unsigned int newsize)
+{
+	elements.resize(newsize);
+}
+
 /* virtual public, overriding as_object::set_member */
 void
-as_array_object::set_member(const tu_stringi& name,
+as_array_object::set_member(const std::string& name,
 		const as_value& val )
 {
-	if ( name == "length" ) 
-	{
-		log_warning("Attempt to assign to Array.length - ignored");
-		return;
-	}
-
 	int index = index_requested(name);
 
 	// if we were sent a valid array index and not a normal member
@@ -359,7 +418,7 @@ std::auto_ptr<as_array_object>
 as_array_object::sorted_indexes(uint8_t flags)
 {
 	assert(flags & as_array_object::fReturnIndexedArray);
-	log_error("Array.sorted_index() method not implemented yet!\n");
+	log_unimpl("Array.sorted_index");
 	return std::auto_ptr<as_array_object>(NULL);
 }
 
@@ -378,43 +437,43 @@ as_array_object::sort(uint8_t flags)
 	switch ( flags )
 	{
 		case 0: // default sorting
-			//log_msg("Default sorting");
+			//log_msg(_("Default sorting"));
 			std::sort(elements.begin(), elements.end(),
 				AsValueLessThen());
 			break;
 
 		case as_array_object::fDescending:
-			//log_msg("Default descending");
+			//log_msg(_("Default descending"));
 			std::sort(elements.begin(), elements.end(),
 				AsValueLessThenDesc());
 			break;
 
 		case as_array_object::fCaseInsensitive: 
-			//log_msg("case insensitive");
+			//log_msg(_("case insensitive"));
 			std::sort(elements.begin(), elements.end(),
 				AsValueLessThenNoCase());
 			break;
 
 		case as_array_object::fCaseInsensitive | as_array_object::fDescending:
-			//log_msg("case insensitive descending");
+			//log_msg(_("case insensitive descending"));
 			std::sort(elements.begin(), elements.end(),
 				AsValueLessThenDescNoCase());
 			break;
 
 		case as_array_object::fNumeric: 
-			//log_msg("numeric");
+			//log_msg(_("numeric"));
 			std::sort(elements.begin(), elements.end(),
 				AsValueLessThenNumeric());
 			break;
 
 		case as_array_object::fNumeric | as_array_object::fDescending:
-			//log_msg("numeric descending");
+			//log_msg(_("numeric descending"));
 			std::sort(elements.begin(), elements.end(),
 				AsValueLessThenDescNumeric());
 			break;
 
 		default:
-			log_error("Unhandled sort flags: %d (0x%X)", flags, flags);
+			log_error(_("Unhandled sort flags: %d (0x%X)"), flags, flags);
 			break;
 	}
 
@@ -424,7 +483,7 @@ as_array_object::sort(uint8_t flags)
 	// to either use std::sort or std::uniq or similar
 	if ( do_unique )
 	{
-		log_msg("Should unique now");
+		log_unimpl(_("array.sort with unique flag"));
 	}
 }
 
@@ -442,25 +501,81 @@ as_array_object::sort(as_function& comparator, uint8_t flags)
 
 }
 
-static void
+static as_value
 array_splice(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	//as_array_object* array = static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
-	log_error("Array.splice() method not implemented yet!\n");
-	fn.result->set_undefined();
+#ifdef GNASH_DEBUG
+	std::stringstream ss;
+	fn.dump_args(ss);
+	log_msg(_("Array(%s).splice(%s) called"), array->toString().c_str(), ss.str().c_str());
+#endif
+
+	if (fn.nargs < 1)
+	{
+		IF_VERBOSE_ASCODING_ERRORS(
+		log_aserror(_("Array.splice() needs at least 1 argument, call ignored"));
+		);
+		return as_value();
+	}
+
+	unsigned origlen = array->size();
+
+	//----------------
+	// Get start offset
+	//----------------
+	unsigned startoffset;
+	int start = fn.arg(0).to_number<int>(&(fn.env()));
+	if ( start < 0 ) start = array->size()+start; // start is negative, so + means -abs()
+	startoffset = iclamp(start, 0, origlen);
+#ifdef GNASH_DEBUG
+	if ( startoffset != start )
+		log_msg(_("Array.splice: start:%d became %u"), start, startoffset);
+#endif
+
+	//----------------
+	// Get length
+	//----------------
+	unsigned len = 0;
+	if (fn.nargs > 1)
+	{
+		int lenval = fn.arg(1).to_number<int>(&(fn.env()));
+		if ( lenval < 0 )
+		{
+			IF_VERBOSE_ASCODING_ERRORS(
+			log_aserror(_("Array.splice(%d,%d): negative length given, call ignored"),
+				start, lenval);
+			);
+			return as_value();
+		}
+		len = iclamp(lenval, 0, origlen-startoffset);
+	}
+
+	//----------------
+	// Get replacement
+	//----------------
+	std::vector<as_value> replace;
+	for (unsigned i=2; i<fn.nargs; ++i)
+	{
+		replace.push_back(fn.arg(i));
+	}
+
+	std::auto_ptr<as_array_object> spliced ( array->splice(startoffset, len, replace) );
+
+	boost::intrusive_ptr<as_object> ret = spliced.release();
+
+	return as_value(ret);
 }
 
-static void
+static as_value
 array_sort(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	as_array_object* array = static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
 	uint8_t flags;
 
-	if ( fn.nargs == 1 && fn.arg(0).get_type() == as_value::NUMBER )
+	if ( fn.nargs == 1 && fn.arg(0).is_number() )
 	{
 		flags=static_cast<uint8_t>(fn.arg(0).to_number());
 	}
@@ -470,151 +585,148 @@ array_sort(const fn_call& fn)
 	}
 	else
 	{
-		log_error("Array.sort(comparator) method not implemented!\n");
-		fn.result->set_undefined();
-		return;
+		log_unimpl("Array.sort(comparator)");
+		return as_value();
 	}
 
 	array->sort(flags);
-	fn.result->set_undefined(); // returns void
-	return;
+	return as_value(); // returns void
 
 }
 
-static void
+static as_value
 array_sortOn(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	//as_array_object* array = static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
+	UNUSED(array);
 
-	log_error("Array.sortOn() method not implemented yet!\n");
-	fn.result->set_undefined();
+	log_unimpl("Array.sortOn()");
+	return as_value();
 }
 
 // Callback to push values to the back of an array
-static void
+static as_value
 array_push(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	as_array_object* array = \
-		static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
 		IF_VERBOSE_ACTION (
-	log_action("calling array push, pushing %d values onto back of array\n",fn.nargs);
+	log_action(_("calling array push, pushing %d values onto back of array"),fn.nargs);
 		);
 
-	for (int i=0;i<fn.nargs;i++)
+	for (unsigned int i=0;i<fn.nargs;i++)
 		array->push(fn.arg(i));
 
-	fn.result->set_int(array->size());
+	return as_value(array->size());
 }
 
 // Callback to push values to the front of an array
-static void
+static as_value
 array_unshift(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	as_array_object* array = \
-		static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
 		IF_VERBOSE_ACTION (
-	log_action("calling array unshift, pushing %d values onto front of array\n",fn.nargs);
+	log_action(_("calling array unshift, pushing %d values onto front of array"), fn.nargs);
 		);
 
-	for (int i=fn.nargs-1;i>=0;i--)
+	for (int i=fn.nargs-1; i>=0; i--)
 		array->unshift(fn.arg(i));
 
-	fn.result->set_int(array->size());
+	return as_value(array->size());
 }
 
 // Callback to pop a value from the back of an array
-static void
+static as_value
 array_pop(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	as_array_object* array = \
-		static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
 	// Get our index, log, then return result
-	(*fn.result) = array->pop();
+	as_value rv = array->pop();
 
-		IF_VERBOSE_ACTION (
-	log_action("calling array pop, result:%s, new array size:%d\n",fn.result->to_string(),array->size());
-		);
+	IF_VERBOSE_ACTION (
+	log_action(_("calling array pop, result:%s, new array size:%d"),
+		rv.to_string().c_str(), array->size());
+	);
+        return rv;
 }
 
 // Callback to pop a value from the front of an array
-static void
+static as_value
 array_shift(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	as_array_object* array = \
-		static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
 	// Get our index, log, then return result
-	(*fn.result) = array->shift();
+	as_value rv = array->shift();
 
-		IF_VERBOSE_ACTION (
-	log_action("calling array shift, result:%s, new array size:%d\n",fn.result->to_string(),array->size());
-		);
+	IF_VERBOSE_ACTION (
+	log_action(_("calling array shift, result:%s, new array size:%d"),
+		rv.to_string().c_str(), array->size());
+	);
+	return rv;
 }
 
 // Callback to reverse the position of the elements in an array
-static void
+static as_value
 array_reverse(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	as_array_object* array = \
-		static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
 	array->reverse();
 
-	fn.result->set_as_object(array);
+	as_value rv(array.get()); 
 
-		IF_VERBOSE_ACTION (
-	log_action("called array reverse, result:%s, new array size:%d\n",fn.result->to_string(),array->size());
-		);
-	
+	IF_VERBOSE_ACTION (
+	log_action(_("called array reverse, result:%s, new array size:%d"),
+		rv.to_string().c_str(), array->size());
+	);
+	return rv;
 }
 
 // Callback to convert array to a string with optional custom separator (default ',')
-static void
+static as_value
 array_join(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	as_array_object* array = \
-		static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
 	std::string separator = ",";
 
 	if (fn.nargs > 0)
 		separator = fn.arg(0).to_string();
 
-	std::string ret = array->join(separator);
+	std::string ret = array->join(separator, &(fn.env()));
 
-	fn.result->set_string(ret.c_str());
+	return as_value(ret.c_str());
+}
+
+static as_value
+array_size(const fn_call& fn)
+{
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
+
+	return as_value(array->size());
 }
 
 // Callback to convert array to a string
-static void
+// TODO CHECKME: rely on Object.toString  ? (
+static as_value
 array_to_string(const fn_call& fn)
 {
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	as_array_object* array = \
-		static_cast<as_array_object*>(fn.this_ptr);
-
-	std::string ret = array->toString();
+	std::string ret = array->toString(&(fn.env()));
 
 		IF_VERBOSE_ACTION
 		(
-	log_action("array_to_string called, nargs = %d, "
-			"this_ptr = %p",
-			fn.nargs, (void*)fn.this_ptr);
-	log_action("to_string result is: %s", ret.c_str());
+	log_action(_("array_to_string called, nargs = %d, "
+			"this_ptr = %p"),
+			fn.nargs, (void*)fn.this_ptr.get());
+	log_action(_("to_string result is: %s"), ret.c_str());
 		);
 
-	fn.result->set_string(ret.c_str());
+	return as_value(ret.c_str());
 }
 
 /// concatenates the elements specified in the parameters with
@@ -622,22 +734,20 @@ array_to_string(const fn_call& fn)
 /// value parameters specify an array, the elements of that
 /// array are concatenated, rather than the array itself. The
 /// array my_array is left unchanged.
-static void
+static as_value
 array_concat(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	as_array_object* array = \
-		static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
 	// use copy ctor
 	as_array_object* newarray = new as_array_object(*array);
 
-	for (int i=0; i<fn.nargs; i++)
+	for (unsigned int i=0; i<fn.nargs; i++)
 	{
 		// Array args get concatenated by elements
-		if ( as_array_object* other = dynamic_cast<as_array_object*>(fn.arg(i).to_object()) )
+		boost::intrusive_ptr<as_array_object> other = boost::dynamic_pointer_cast<as_array_object>(fn.arg(1).to_object());
+		if ( other )
 		{
-			assert(other);
 			newarray->concat(*other);
 		}
 		else
@@ -646,25 +756,26 @@ array_concat(const fn_call& fn)
 		}
 	}
 
-	fn.result->set_as_object(newarray);		
+	return as_value(newarray);		
 }
 
 // Callback to slice part of an array to a new array
 // without changing the original
-static void
+static as_value
 array_slice(const fn_call& fn)
 {
-	assert(dynamic_cast<as_array_object*>(fn.this_ptr));
-	as_array_object* array = \
-		static_cast<as_array_object*>(fn.this_ptr);
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
 
 	// start and end index of the part we're slicing
 	int startindex, endindex;
 
 	if (fn.nargs > 2)
 	{
-		log_error("More than 2 arguments sent to slice, and I don't know what to do with them!\n"
-			"Ignoring them as we continue...\n");
+		IF_VERBOSE_ASCODING_ERRORS(
+		log_aserror(_("More than 2 arguments to Array.slice, "
+			"and I don't know what to do with them.  "
+			"Ignoring them"));
+		);
 	}
 
 	// They passed no arguments: simply duplicate the array
@@ -672,8 +783,7 @@ array_slice(const fn_call& fn)
 	if (fn.nargs < 1)
 	{
 		as_array_object* newarray = new as_array_object(*array);
-		fn.result->set_as_object(newarray);
-		return;
+		return as_value(newarray);
 	}
 
 
@@ -682,64 +792,66 @@ array_slice(const fn_call& fn)
 	// if the index is negative, it means "places from the end"
 	// where -1 is the last element
 	if (startindex < 0) startindex = startindex + array->size();
-	// if it's still negative, this is a problem
-	if (startindex < 0 || (unsigned int)startindex > array->size())
-	{
-		log_error("Bad startindex sent to array_slice! startindex: %s, Length: %d. "
-			"Ignoring call.",
-			fn.arg(0).to_string(),array->size());
-		return;				
-	}
+
 	// if we sent at least two arguments, setup endindex
 	if (fn.nargs >= 2)
 	{
 		endindex = int(fn.arg(1).to_number());
+
 		// if the index is negative, it means
 		// "places from the end" where -1 is the last element
 		if (endindex < 0) endindex = endindex + array->size();
-		// the endindex is non-inclusive, so add 1
-		endindex++;
-		if (endindex < 0)
-		{
-			log_error("bad endindex sent to array_slice! endindex: %s, length: %d"
-				"Ignoring call.",
-				fn.arg(1).to_string(),array->size());
-			return;				
-		}
-		// If they overshoot the end of the array,
-		// just copy to the end
-		if ((unsigned int)endindex > array->size() + 1)
-			endindex = array->size() + 1;
 	}
 	else
 	{
-		// They didn't specify where to end, so choose the end of the array
-		endindex = array->size() + 1;
+		// They didn't specify where to end,
+		// so choose the end of the array
+		endindex = array->size();
 	}
 
+	if ( startindex < 0 ) startindex = 0;
+	else if ( static_cast<size_t>(startindex)  > array->size() ) startindex = array->size();
+
+	if ( endindex < 1 ) endindex = 1;
+	else if ( static_cast<size_t>(endindex)  > array->size() ) endindex = array->size();
+
 	std::auto_ptr<as_array_object> newarray(array->slice(
 		startindex, endindex));
 
-	fn.result->set_as_object(newarray.release());		
+	return as_value(newarray.release());		
 
 }
 
-void
+static as_value
+array_length(const fn_call& fn)
+{
+	boost::intrusive_ptr<as_array_object> array = ensureType<as_array_object>(fn.this_ptr);
+
+	if ( fn.nargs ) // setter
+	{
+		array->resize(unsigned(fn.arg(0).to_number(&(fn.env()))));
+		return as_value();
+	}
+	else // getter
+	{
+		return as_value(array->size());
+	}
+}
+
+as_value
 array_new(const fn_call& fn)
 {
 	IF_VERBOSE_ACTION (
-		log_action("array_new called, nargs = %d", fn.nargs);
+		log_action(_("array_new called, nargs = %d"), fn.nargs);
 	);
 
-	//smart_ptr<as_array_object>	ao = new as_array_object;
-	as_array_object* ao = new as_array_object;
+	boost::intrusive_ptr<as_array_object>	ao = new as_array_object;
 
 	if (fn.nargs == 0)
 	{
 		// Empty array.
 	}
-	else if (fn.nargs == 1
-		 && fn.arg(0).get_type() == as_value::NUMBER)
+	else if (fn.nargs == 1 && fn.arg(0).is_number() )
 	{
 		// Create an empty array with the given number of undefined elements.
 		//
@@ -749,64 +861,71 @@ array_new(const fn_call& fn)
 		for (int i = 0; i < int(fn.arg(0).to_number()); i++)
 		{
 			index_number.set_int(i);
-			ao->set_member(index_number.to_string(), null_value);
+			ao->set_member(index_number.to_string().c_str(), null_value);
 		}
 	}
 	else
 	{
 		// Use the arguments as initializers.
 		as_value	index_number;
-		for (int i = 0; i < fn.nargs; i++)
+		for (unsigned int i = 0; i < fn.nargs; i++)
 		{
 			ao->push(fn.arg(i));
 		}
 	}
 
 	IF_VERBOSE_ACTION (
-		log_action("array_new setting object %p in result", (void*)ao);
+		log_action(_("array_new setting object %p in result"), (void*)ao.get());
 	);
 
-	//fn.result->set_as_object(ao.get_ptr());
-	fn.result->set_as_object(ao);
+	return as_value(ao.get());
+	//return as_value(ao);
+}
+
+static void
+attachArrayProperties(as_object& proto)
+{
+	boost::intrusive_ptr<builtin_function> gettersetter;
+
+	gettersetter = new builtin_function(&array_length, NULL);
+	proto.init_property("length", *gettersetter, *gettersetter);
 }
 
 static void
 attachArrayInterface(as_object& proto)
 {
-	// we don't need an explicit member here,
-	// we will be handling 'length' requests
-	// within overridden get_member()
-	//proto->set_member("length", &array_length);
 
-	proto.set_member("join", &array_join);
-	proto.set_member("concat", &array_concat);
-	proto.set_member("slice", &array_slice);
-	proto.set_member("push", &array_push);
-	proto.set_member("unshift", &array_unshift);
-	proto.set_member("pop", &array_pop);
-	proto.set_member("shift", &array_shift);
-	proto.set_member("splice", &array_splice);
-	proto.set_member("sort", &array_sort);
-	proto.set_member("sortOn", &array_sortOn);
-	proto.set_member("reverse", &array_reverse);
-	proto.set_member("toString", &array_to_string);
-	proto.set_member("CASEINSENSITIVE", as_array_object::fCaseInsensitive);
-	proto.set_member("DESCENDING", as_array_object::fDescending);
-	proto.set_member("UNIQUESORT", as_array_object::fUniqueSort);
-	proto.set_member("RETURNINDEXEDARRAY", as_array_object::fReturnIndexedArray);
-	proto.set_member("NUMERIC", as_array_object::fNumeric);
+	proto.init_member("join", new builtin_function(array_join));
+	proto.init_member("concat", new builtin_function(array_concat));
+	proto.init_member("slice", new builtin_function(array_slice));
+	proto.init_member("push", new builtin_function(array_push));
+	proto.init_member("unshift", new builtin_function(array_unshift));
+	proto.init_member("pop", new builtin_function(array_pop));
+	proto.init_member("shift", new builtin_function(array_shift));
+	proto.init_member("splice", new builtin_function(array_splice));
+	proto.init_member("sort", new builtin_function(array_sort));
+	proto.init_member("size", new builtin_function(array_size));
+	proto.init_member("sortOn", new builtin_function(array_sortOn));
+	proto.init_member("reverse", new builtin_function(array_reverse));
+	proto.init_member("toString", new builtin_function(array_to_string));
+
+	proto.init_member("CASEINSENSITIVE", as_array_object::fCaseInsensitive);
+	proto.init_member("DESCENDING", as_array_object::fDescending);
+	proto.init_member("UNIQUESORT", as_array_object::fUniqueSort);
+	proto.init_member("RETURNINDEXEDARRAY", as_array_object::fReturnIndexedArray);
+	proto.init_member("NUMERIC", as_array_object::fNumeric);
 }
 
 static as_object*
 getArrayInterface()
 {
-	static smart_ptr<as_object> proto = NULL;
+	static boost::intrusive_ptr<as_object> proto = NULL;
 	if ( proto == NULL )
 	{
 		proto = new as_object();
 		attachArrayInterface(*proto);
 	}
-	return proto.get_ptr();
+	return proto.get();
 }
 
 // this registers the "Array" member on a "Global"
@@ -818,7 +937,7 @@ void
 array_class_init(as_object& glob)
 {
 	// This is going to be the global Array "class"/"function"
-	static smart_ptr<as_function> ar=NULL;
+	static boost::intrusive_ptr<as_function> ar=NULL;
 
 	if ( ar == NULL )
 	{
@@ -826,12 +945,18 @@ array_class_init(as_object& glob)
 
 		// We replicate interface to the Array class itself
 		attachArrayInterface(*ar);
-
 	}
 
 	// Register _global.Array
-	glob.set_member("Array", ar.get_ptr());
+	glob.init_member("Array", ar.get());
 }
 
 
-}
+} // end of gnash namespace
+
+
+// Local Variables:
+// mode: C++
+// indent-tabs-mode: t
+// End:
+
