@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,6 +10,7 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -21,9 +22,10 @@
 #ifndef GNASH_ARRAY_H
 #define GNASH_ARRAY_H
 
-//#include "action.h"
 #include "as_object.h" // for inheritance
+
 #include <deque>
+#include <vector>
 #include <memory> // for auto_ptr
 
 #include <string>
@@ -67,9 +69,9 @@ public:
 
 	as_array_object(const as_array_object& other);
 
-	void push(as_value& val);
+	void push(const as_value& val);
 
-	void unshift(as_value& val);
+	void unshift(const as_value& val);
 
 	as_value shift();
 
@@ -79,19 +81,75 @@ public:
 
 	void reverse();
 
-	std::string join(const std::string& separator);
+	/// @param env
+	///	If not-null will be used to properl invoke the toString()
+	///	method against member values.
+	///
+	std::string join(const std::string& separator, as_environment* env) const;
 
-	std::string toString();
+	/// @param env
+	///	If not-null will be used to properly invoke the toString()
+	///	method against member values.
+	///
+	std::string toString(as_environment* env=NULL) const;
+
+	// override from as_object
+	std::string get_text_value() const
+	{
+		return toString();
+	}
 
 	unsigned int size() const;
 
-	//void resize(unsigned int);
+	void resize(unsigned int);
 
 	void concat(const as_array_object& other);
 
+	/// \brief
+	/// Return a newly created array containing elements
+	/// from 'start' up to but not including 'end'.
+	//
+	///
+	/// NOTE: assertions are:
+	///
+	///	assert(one_past_end >= start);
+	///	assert(one_past_end <= size());
+	///	assert(start <= size());
+	///
+	/// @param start
+	///	index to first element to include in result
+	///	0-based index.
+	///
+	/// @param one_past_end
+	///	index to one-past element to include in result
+	///	0-based index.
+	///
 	std::auto_ptr<as_array_object> slice(
 		unsigned int start, unsigned int one_past_end);
 
+	/// \brief
+	/// Substitute 'len' elements from 'start' with elements from
+	/// the given array.
+	//
+	/// NOTE: assertions are:
+	///
+	///	assert(len <= size()-start);
+	///	assert(start <= size());
+	///
+	/// @param start
+	///	0-index based offset of first element to replace.
+	///
+	/// @param len
+	///	Number of elements to replace.
+	///
+	/// @param replacement
+	///	The element to use as replacement
+	///
+	/// TODO: should we return by intrusive_ptr instead ?
+	///
+	std::auto_ptr<as_array_object> splice(unsigned start, unsigned len,
+			const std::vector<as_value>& replacement);
+
 	/// Sort the array, using given values comparator
 	void sort(as_function& comparator, uint8_t flags=0);
 
@@ -105,20 +163,24 @@ public:
 	std::auto_ptr<as_array_object> sorted_indexes(as_function& comparator, uint8_t flags);
 
 	/// Overridden to provide 'length' member
-	virtual bool get_member(const tu_stringi& name, as_value* val);
+	//
+	/// TODO: use a property for handling 'length'
+	virtual bool get_member(const std::string& name, as_value* val);
 
 	/// Overridden to provide array[#]=x semantic
-	virtual void set_member(const tu_stringi& name,
+	virtual void set_member(const std::string& name,
 		const as_value& val );
 
 private:
 
-	std::deque<as_value> elements;
+	typedef std::deque<as_value> container;
+
+	container elements;
 
 	// this function is used internally by set_member and get_member
 	// it takes a string that is the member name of the array and returns -1
 	// if the string does not refer to an index, or an appropriate int if the string does refer to an index
-	int index_requested(const tu_stringi& name);
+	int index_requested(const std::string& name);
 
 };
 
@@ -129,7 +191,7 @@ void array_class_init(as_object& global);
 
 /// Constructor for ActionScript class Array.
 // needed by SWFHandlers::ActionInitArray
-void	array_new(const fn_call& fn);
+as_value	array_new(const fn_call& fn);
 
 }
 
