@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,6 +10,7 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -23,12 +24,10 @@
 #ifndef GNASH_MOVIECLIPLOADER_H
 #define GNASH_MOVIECLIPLOADER_H
 
-#include "button_character_instance.h" // for mouse_state enum
-#include "action.h"
-//#include "impl.h"
-
 namespace gnash {
 
+class as_object;
+
 /// Initialize the global MovieClipLoader class
 void moviecliploader_class_init(as_object& global);
 
