@@ -1,25 +1,338 @@
-// text.cpp	-- Thatcher Ulrich <tu@tulrich.com> 2003
+// edit_text_character.cpp:  User-editable text regions, for Gnash.
+//
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+// 
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
+// 
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+//
 
-// This source code has been donated to the Public Domain.  Do
-// whatever you want with it.
+/* $Id: edit_text_character.cpp,v 1.62.4.1 2007/06/08 08:18:02 udog Exp $ */
 
-/* $Id: edit_text_character.cpp,v 1.22.2.9 2006/11/15 12:56:42 strk Exp $ */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "utf8.h"
 #include "log.h"
-#include "render.h"
+#include "render.h" // for display()
 #include "movie_definition.h" // to extract version info
 #include "sprite_instance.h"
 #include "edit_text_character.h"
-#include "Key.h"
-#include "movie_root.h"	
+#include "Key.h" // for keyboard events
+#include "movie_root.h"	 // for killing focus
 #include "as_environment.h" // for parse_path
+#include "action.h" // for as_standard_member enum
+#include "VM.h"
+#include "builtin_function.h" // for getter/setter properties
+#include "font.h" // for using the _font member
+#include "VM.h"
 
 #include <algorithm>
 #include <string>
+#include <boost/algorithm/string/case_conv.hpp>
+
+// Text fields have a fixed 2 pixel padding for each side (regardless of border)
+#define PADDING_TWIPS 40.0f
+
 
 namespace gnash {
 
+// Forward declarations
+static as_value textfield_get_variable(const fn_call& fn);
+static as_value textfield_set_variable(const fn_call& fn);
+static as_value textfield_setTextFormat(const fn_call& fn);
+static as_value textfield_getTextFormat(const fn_call& fn);
+static as_value textfield_setNewTextFormat(const fn_call& fn);
+static as_value textfield_getNewTextFormat(const fn_call& fn);
+static as_value textfield_addListener(const fn_call& fn);
+static as_value textfield_removeListener(const fn_call& fn);
+
+static as_value textfield_getDepth(const fn_call& fn);
+static as_value textfield_getFontList(const fn_call& fn);
+static as_value textfield_removeTextField(const fn_call& fn);
+static as_value textfield_replaceSel(const fn_call& fn);
+static as_value textfield_replaceText(const fn_call& fn);
+static as_object* getTextFieldInterface();
+
+
+//
+// TextField interface functions
+//
+
+static as_value
+textfield_get_variable(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+
+	return as_value(text->get_variable_name());
+
+}
+
+static as_value
+textfield_set_variable(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+
+	assert ( fn.nargs > 0 );
+	const std::string& newname = fn.arg(0).to_string(&fn.env());
+
+	text->set_variable_name(newname);
+
+	return as_value();
+}
+
+static as_value
+textfield_setTextFormat(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.setTextFormat()");
+		warned = true;
+	}
+
+	return as_value();
+
+}
+
+static as_value
+textfield_addListener(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.addListener()");
+		warned = true;
+	}
+
+	return as_value();
+}
+
+static as_value
+textfield_removeListener(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.removeListener()");
+		warned = true;
+	}
+
+	return as_value();
+}
+
+static as_value
+textfield_setNewTextFormat(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.setNewTextFormat()");
+		warned = true;
+	}
+
+	return as_value();
+}
+
+static as_value
+textfield_getDepth(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.getDepth()");
+		warned = true;
+	}
+
+	return as_value();
+}
+
+static as_value
+textfield_getFontList(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.getFontList()");
+		warned = true;
+	}
+
+	return as_value();
+}
+
+static as_value
+textfield_getNewTextFormat(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.getNewTextFormat()");
+		warned = true;
+	}
+
+	return as_value();
+}
+
+static as_value
+textfield_getTextFormat(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.getTextFormat()");
+		warned = true;
+	}
+
+	return as_value();
+}
+
+static as_value
+textfield_replaceSel(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.replaceSel()");
+		warned = true;
+	}
+
+	return as_value();
+}
+
+static as_value
+textfield_replaceText(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.replaceText()");
+		warned = true;
+	}
+
+	return as_value();
+}
+
+static as_value
+textfield_removeTextField(const fn_call& fn)
+{
+	boost::intrusive_ptr<edit_text_character> text = ensureType<edit_text_character>(fn.this_ptr);
+	UNUSED(text);
+
+	static bool warned = false;
+	if ( ! warned ) {
+		log_unimpl("TextField.removeTextField()");
+		warned = true;
+	}
+
+	return as_value();
+}
+
+static as_value
+textfield_ctor(const fn_call& /* fn */)
+{
+	boost::intrusive_ptr<as_object> obj = new as_object(getTextFieldInterface());
+	return as_value(obj);
+}
+
+//
+// TextField interface initialization
+//
+
+static void
+attachTextFieldInterface(as_object& o)
+{
+	int target_version = o.getVM().getSWFVersion();
+
+	// SWF5 or higher
+	if ( target_version  < 6 ) return;
+
+	// SWF6 or higher
+	boost::intrusive_ptr<builtin_function> variable_getter(new builtin_function(&textfield_get_variable, NULL));
+	boost::intrusive_ptr<builtin_function> variable_setter(new builtin_function(&textfield_set_variable, NULL));
+	o.init_property("variable", *variable_getter, *variable_setter);
+	o.init_member("setTextFormat", new builtin_function(textfield_setTextFormat));
+	o.init_member("getTextFormat", new builtin_function(textfield_getTextFormat));
+	o.init_member("addListener", new builtin_function(textfield_addListener));
+	o.init_member("removeListener", new builtin_function(textfield_removeListener));
+	o.init_member("setNewTextFormat", new builtin_function(textfield_setNewTextFormat));
+	o.init_member("getNewTextFormat", new builtin_function(textfield_getNewTextFormat));
+	o.init_member("getNewTextFormat", new builtin_function(textfield_getNewTextFormat));
+	o.init_member("getDepth", new builtin_function(textfield_getDepth));
+	o.init_member("removeTextField", new builtin_function(textfield_removeTextField));
+	o.init_member("replaceSel", new builtin_function(textfield_replaceSel));
+	if ( target_version  < 7 ) return;
+
+	// SWF7 or higher
+	o.init_member("replaceText", new builtin_function(textfield_replaceText));
+	if ( target_version  < 8 ) return;
+
+}
+
+static void
+attachTextFieldStaticMembers(as_object& o)
+{
+	int target_version = o.getVM().getSWFVersion();
+
+	// SWF5 or higher
+	if ( target_version  < 6 ) return;
+
+	// SWF6 or higher
+	o.init_member("getFontList", new builtin_function(textfield_getFontList));
+	if ( target_version  < 7 ) return;
+
+	// SWF7 or higher
+	if ( target_version  < 8 ) return;
+
+}
+
+static as_object*
+getTextFieldInterface()
+{
+	static boost::intrusive_ptr<as_object> proto;
+	if ( proto == NULL )
+	{
+		proto = new as_object();
+		attachTextFieldInterface(*proto);
+		proto->init_member("constructor", new builtin_function(textfield_ctor)); 
+	}
+	return proto.get();
+}
+
+//
+// edit_text_character class
+//
+
 edit_text_character::edit_text_character(character* parent,
 		edit_text_character_def* def, int id)
 	:
@@ -27,14 +340,17 @@ edit_text_character::edit_text_character(character* parent,
 	m_def(def),
 	_font(0),
 	m_has_focus(false),
-	m_cursor(0),
+	m_cursor(0u),
 	m_xcursor(0.0f),
 	m_ycursor(0.0f),
-	_text_variable_registered(false)
+	_text_variable_registered(false),
+	_variable_name(m_def->get_variable_name())
 {
 	assert(parent);
 	assert(m_def);
 
+	set_prototype(getTextFieldInterface());
+
 	// WARNING! remember to set the font *before* setting text value!
 	set_font( m_def->get_font() );
 
@@ -56,9 +372,8 @@ edit_text_character::~edit_text_character()
 	on_event(event_id::KILLFOCUS);
 }
 
-movie_root*	edit_text_character::get_root() { return get_parent()->get_root(); }
-
-void edit_text_character::show_cursor()
+void
+edit_text_character::show_cursor()
 {
 	uint16_t x = static_cast<uint16_t>(m_xcursor);
 	uint16_t y = static_cast<uint16_t>(m_ycursor);
@@ -71,7 +386,6 @@ void edit_text_character::show_cursor()
 	box[3] = y + h;
 	
 	render::draw_line_strip(box, 2, rgba(0,0,0,255));	// draw line
-
 }
 
 void
@@ -81,6 +395,8 @@ edit_text_character::display()
 
 	registerTextVariable();
 
+	rect def_bounds = m_def->get_bounds();
+
 	if (m_def->has_border())
 	{
 		matrix	mat = get_world_matrix();
@@ -93,7 +409,6 @@ edit_text_character::display()
 		//mat.print();
 
 		point	coords[4];
-		const rect def_bounds = m_def->get_bounds();
 		
 		coords[0] = def_bounds.get_corner(0);
 		coords[1] = def_bounds.get_corner(1);
@@ -135,8 +450,20 @@ edit_text_character::display()
 	}
 
 	// Draw our actual text.
-	display_glyph_records(matrix::identity, this, m_text_glyph_records,
-		m_def->get_root_def());
+	// Using a matrix to translate to def bounds seems an hack to me.
+	// A cleaner implementation is likely correctly setting the
+	// m_x_offset and m_y_offset memebers in glyph records.
+	// Anyway, see bug #17954 for a testcase.
+	matrix m;
+
+	if ( ! def_bounds.is_null() && ! def_bounds.is_world() )
+	{
+		m.concatenate_translation(def_bounds.get_x_min(), def_bounds.get_y_min());
+	}
+	
+	
+	display_glyph_records(m, this, m_text_glyph_records,
+			      m_def->get_root_def());
 
 	if (m_has_focus)
 	{
@@ -148,19 +475,24 @@ edit_text_character::display()
 }
 
 
-	void edit_text_character::get_invalidated_bounds(rect* bounds, bool force) {
-	
-    if (!force && !m_invalidated) return; // no need to redraw
+void
+edit_text_character::add_invalidated_bounds(InvalidatedRanges& ranges, 
+	bool force)
+{
+	if (!force && !m_invalidated) return; // no need to redraw
     
-    bounds->expand_to_rect(m_old_invalidated_bounds);	
-	
-    bounds->expand_to_transformed_rect(get_world_matrix(), 
-      m_def->get_bound());            
-	  
-  }
+	ranges.add(m_old_invalidated_ranges);
 
+	rect bounds;	
 
-bool edit_text_character::on_event(event_id id)
+	bounds.expand_to_transformed_rect(get_world_matrix(),
+    				       m_def->get_bound());
+
+	ranges.add(bounds.getRange());            
+}
+
+bool
+edit_text_character::on_event(const event_id& id)
 {
 	if (m_def->get_readonly() == true)
 	{
@@ -173,9 +505,9 @@ bool edit_text_character::on_event(event_id id)
 		{
 			if (m_has_focus == false)
 			{
-				get_root()->add_keypress_listener(this);
+				_vm.getRoot().add_key_listener(this);
 				m_has_focus = true;
-				m_cursor = m_text.size();
+				m_cursor = _text.size();
 				format_text();
 			}
 			break;
@@ -185,8 +517,9 @@ bool edit_text_character::on_event(event_id id)
 		{
 			if (m_has_focus == true)
 			{
-				get_root()->set_active_entity(NULL);
-				get_root()->remove_keypress_listener(this);
+				movie_root& root = _vm.getRoot();
+				root.set_active_entity(NULL);
+				root.remove_key_listener(this);
 				m_has_focus = false;
 				format_text();
 			}
@@ -195,12 +528,12 @@ bool edit_text_character::on_event(event_id id)
 
 		case event_id::KEY_PRESS:
 		{
-			std::string s(m_text.c_str());
+			std::string s(_text);
 			std::string c;
 			c = (char) id.m_key_code;
 
-			// may be m_text is changed in ActionScript
-			m_cursor = imin(m_cursor, m_text.size());
+			// may be _text is changed in ActionScript
+			m_cursor = imin(m_cursor, _text.size());
 
 			switch (c[0])
 			{
@@ -214,7 +547,7 @@ bool edit_text_character::on_event(event_id id)
 					break;
 
 				case key::DELETEKEY:
-					if (s.size() > (unsigned int)m_cursor)
+					if (s.size() > m_cursor)
 					{
 						s.erase(m_cursor, 1);
 						set_text_value(s.c_str());
@@ -234,7 +567,7 @@ bool edit_text_character::on_event(event_id id)
 				case key::END:
 				case key::PGDN:
 				case key::DOWN:
-					m_cursor = m_text.size();
+					m_cursor = _text.size();
 					format_text();
 					break;
 
@@ -244,7 +577,7 @@ bool edit_text_character::on_event(event_id id)
 					break;
 
 				case key::RIGHT:
-					m_cursor = m_cursor < m_text.size() ? m_cursor + 1 : m_text.size();
+					m_cursor = m_cursor < _text.size() ? m_cursor + 1 : _text.size();
 					format_text();
 					break;
 
@@ -264,25 +597,29 @@ bool edit_text_character::on_event(event_id id)
 	return true;
 }
 
-movie*	edit_text_character::get_topmost_mouse_entity(float x, float y)
+character*
+edit_text_character::get_topmost_mouse_entity(float x, float y)
 {
+	//log_msg("get_topmost_mouse_entity called on edit_text_character %p, labeled '%s'", (void*)this, get_text_value().c_str());
+
 	if (get_visible() == false)
 	{
 		return NULL;
 	}
-
+	
+	// shouldn't this be !can_handle_mouse_event() instead ?
 	if (m_def->get_no_select())
-  	{
+	{
 		// not selectable, so don't catch mouse events!
 		return NULL;
 	}
-  	 
+
 	matrix	m = get_matrix();
 		
 	point	p;
 	m.transform_by_inverse(&p, point(x, y));
 
-	const rect def_bounds = m_def->get_bounds();
+	const rect& def_bounds = m_def->get_bounds();
 	if (def_bounds.point_test(p.m_x, p.m_y))
 	{
 		return this;
@@ -291,27 +628,32 @@ movie*	edit_text_character::get_topmost_mouse_entity(float x, float y)
 }
 
 void
-edit_text_character::set_text_value(const char* new_text)
+edit_text_character::set_text_value(const char* new_text_cstr)
 {
-	if (m_text == new_text)
+	std::string new_text;
+	if ( new_text_cstr ) new_text = new_text_cstr;
+
+	if (_text == new_text)
 	{
 		return;
 	}
 
 	set_invalidated();
 
-	m_text = new_text;
+	_text = new_text;
 	if (m_def->get_max_length() > 0
-	    && m_text.length() > m_def->get_max_length() )
+	    && _text.length() > m_def->get_max_length() )
 	{
-		m_text.resize(m_def->get_max_length());
+		_text.resize(m_def->get_max_length());
 	}
 
 	format_text();
-	
+
+	//log_msg(_("Text set to %s"), new_text.c_str());
+
 }
 
-const char*
+std::string
 edit_text_character::get_text_value() const
 {
 	// we need the const_cast here because registerTextVariable
@@ -322,14 +664,19 @@ edit_text_character::get_text_value() const
 	// with a pre-existing value.
 	const_cast<edit_text_character*>(this)->registerTextVariable();
 
-	return m_text.c_str();
+	return _text;
 }
 
 void
-edit_text_character::set_member(const tu_stringi& name,
+edit_text_character::set_member(const std::string& name,
 		const as_value& val)
 {
-	// @@ TODO need to inherit basic stuff like _x, _y, _xscale, _yscale etc
+	//log_msg("edit_text_character.set_member(%s, %s)", name.c_str(), val.to_string());
+
+	// FIXME: Turn all standard members into getter/setter properties
+	//        of the TextField class. See attachTextFieldInterface()
+	// @@ TODO need to inherit basic stuff like _x, _y, _xscale, _yscale etc ?
+
 
 	as_standard_member	std_member = get_standard_member(name);
 	switch (std_member)
@@ -341,7 +688,7 @@ edit_text_character::set_member(const tu_stringi& name,
 		//if (name == "text")
 	{
 		int version = get_parent()->get_movie_definition()->get_version();
-		set_text_value(val.to_tu_string_versioned(version).c_str());
+		set_text_value(val.to_string_versioned(version).c_str());
 		return;
 	}
 	case M_X:
@@ -398,12 +745,19 @@ edit_text_character::set_member(const tu_stringi& name,
 	}
 	// @@ TODO see TextField members in Flash MX docs
 	}	// end switch
+
+	set_member_default(name, val);
 }
 
 bool
-edit_text_character::get_member(const tu_stringi& name, as_value* val)
+edit_text_character::get_member(const std::string& name, as_value* val)
 {
-	as_standard_member	std_member = get_standard_member(name);
+	//log_msg("edit_text_character.get_member(%s)", name.c_str());
+
+	// FIXME: Turn all standard members into getter/setter properties
+	//        of the TextField class. See attachTextFieldInterface()
+	
+	as_standard_member std_member = get_standard_member(name);
 	switch (std_member)
 	{
 	default:
@@ -412,7 +766,7 @@ edit_text_character::get_member(const tu_stringi& name, as_value* val)
 	case M_TEXT:
 		//if (name == "text")
 	{
-		val->set_tu_string(get_text_value());
+		val->set_string(get_text_value());
 		return true;
 	}
 	case M_VISIBLE:
@@ -478,31 +832,27 @@ edit_text_character::get_member(const tu_stringi& name, as_value* val)
 	}
 	}	// end switch
 
-	return false;
+	return get_member_default(name, val);
 }
 	
-// @@ WIDTH_FUDGE is a total fudge to make it match the Flash player!  Maybe
-// we have a bug?
-#define WIDTH_FUDGE 80.0f
-
 
 float
 edit_text_character::align_line(
 		edit_text_character_def::alignment align,
 		int last_line_start_record, float x)
 {
-	GNASH_REPORT_FUNCTION;
+	//GNASH_REPORT_FUNCTION;
 	assert(m_def);
 
 	float	extra_space = (m_def->width() -
-			m_def->get_right_margin()) - x - WIDTH_FUDGE;
+			m_def->get_right_margin()) - x - PADDING_TWIPS;
 
 	//assert(extra_space >= 0.0f);
 	if (extra_space <= 0.0f)
 	{
-		log_warning("TextField text doesn't fit in it's boundaries");
-		log_warning(" m_def->width() == %g", m_def->width());
-		log_warning(" m_def->get_right_margin() == %d", m_def->get_right_margin());
+		log_error(_("TextField text doesn't fit in its boundaries: "
+			    "width %g, margin %d"),
+			    m_def->width(), m_def->get_right_margin());
 		return 0.0f;
 	}
 
@@ -550,8 +900,8 @@ edit_text_character::format_text()
 {
 	m_text_glyph_records.resize(0);
 
-	// nothing to do if we have no text
-	if ( m_text.length() < 1 ) return;
+	// nothing more to do if text is empty
+	if ( _text.empty() ) return;
 
 	// FIXME: I don't think we should query the definition
 	// to find the appropriate font to use, as ActionScript
@@ -559,7 +909,7 @@ edit_text_character::format_text()
 	//
 	if (_font == NULL)
 	{
-		log_error("No font for edit_text_character! [%s:%d]\n",
+		log_error(_("No font for edit_text_character! [%s:%d]"),
 			__FILE__, __LINE__);
 		return;
 	}
@@ -587,12 +937,20 @@ edit_text_character::format_text()
 
 		if (_font != newfont)
 		{
-			log_error("error: substituting font!  font '%s' has no glyphs, using font '%s'\n",
+			log_error(_("substituting font!  font '%s' has "
+				  "no glyphs, using font '%s'"),
 				  fontlib::get_font_name(_font),
-				  fontlib::get_font_name(newfont));
+				  fontlib::get_font_name(newfont)
+			);
 
 			_font = newfont;
 		}
+		else
+		{
+			log_error(_("Current font has no glyphs and I couldn't"
+				  " find another font with glyphs... :("));
+		}
+
 	}
 
 
@@ -601,8 +959,8 @@ edit_text_character::format_text()
 	text_glyph_record	rec;	// one to work on
 	rec.m_style.m_font = _font;
 	rec.m_style.m_color = m_def->get_text_color();
-	rec.m_style.m_x_offset = std::max(0, m_def->get_left_margin() + m_def->get_indent());
-	rec.m_style.m_y_offset = m_def->get_font_height()
+	rec.m_style.m_x_offset = PADDING_TWIPS + std::max(0, m_def->get_left_margin() + m_def->get_indent());
+	rec.m_style.m_y_offset = PADDING_TWIPS + m_def->get_font_height()
 		+ (_font->get_leading() - _font->get_descent()) * scale;
 	rec.m_style.m_text_height = m_def->get_font_height();
 	rec.m_style.m_has_x_offset = true;
@@ -610,7 +968,7 @@ edit_text_character::format_text()
 
 	float	x = rec.m_style.m_x_offset;
 	float	y = rec.m_style.m_y_offset;
-
+	
 
 	// Start the bbox at the upper-left corner of the first glyph.
 	reset_bounding_box(x, y - _font->get_descent() * scale + m_def->get_font_height());
@@ -622,12 +980,12 @@ edit_text_character::format_text()
 	int	last_space_glyph = -1;
 	int	last_line_start_record = 0;
 
-	int character_idx = 0;
+	unsigned int character_idx = 0;
 	m_xcursor = x;
 	m_ycursor = y;
 
-	assert ( m_text.length() > 0 );
-	const char*	text = &m_text[0];
+	assert(! _text.empty() );
+	const char*	text = &_text[0]; 
 	while (uint32_t code = utf8::decode_next_unicode_character(&text))
 	{
 // @@ try to truncate overflow text??
@@ -663,7 +1021,8 @@ edit_text_character::format_text()
 			align_line(m_def->get_alignment(), last_line_start_record, x);
 
 			// new paragraphs get the indent.
-			x = std::max(0, m_def->get_left_margin() + m_def->get_indent());
+			x = std::max(0, m_def->get_left_margin() + m_def->get_indent()) + 
+			  PADDING_TWIPS;
 			y += m_def->get_font_height() + leading;
 
 			// Start a new record on the next line.
@@ -708,33 +1067,64 @@ edit_text_character::format_text()
 			continue;
 		}
 
+		if (code == 9) // tab (ASCII HT)
+		{
+			int index = _font->get_glyph_index(32); // ascii SPACE
+			if ( index == -1 )
+			{
+				IF_VERBOSE_MALFORMED_SWF (
+				  log_error(_("%s -- missing glyph for space char (needed for TAB). "
+					    " Make sure character shapes for font %s are being exported "
+					    "into your SWF file."),
+					    __PRETTY_FUNCTION__,
+					    _font->get_name());
+				);
+			}
+			else
+			{
+				text_glyph_record::glyph_entry	ge;
+				ge.m_glyph_index = index;
+				ge.m_glyph_advance = scale * _font->get_advance(index);
+
+				const int tabstop=8;
+				rec.m_glyphs.insert(rec.m_glyphs.end(), tabstop, ge);
+				x += ge.m_glyph_advance*tabstop;
+			}
+			goto after_x_advance;
+		}
+
 		// Remember where word breaks occur.
 		if (code == 32)
 		{
 			last_space_glyph = rec.m_glyphs.size();
 		}
 
+		{ // need a sub-scope to avoid the 'goto' in TAB handling to cross
+		  // initialization of the 'index' variable
 		int index = _font->get_glyph_index((uint16_t) code);
-		if (index == -1)
-		{
-			// error -- missing glyph!
-			
-			// Log an error, but don't log too many times.
-			static int	s_log_count = 0;
-			if (s_log_count < 10)
-			{
-				s_log_count++;
-				log_warning("%s\n\t -- missing glyph for char %d\n"
-					    "-- make sure character shapes for font %s are being exported "
-					  "into your SWF file!\n",
-					    __PRETTY_FUNCTION__,
-					    code,
-					    _font->get_name());
-			}
+		IF_VERBOSE_MALFORMED_SWF (
+		    if (index == -1)
+		    {
+			    // error -- missing glyph!
+			    
+			    // Log an error, but don't log too many times.
+			    static int	s_log_count = 0;
+			    if (s_log_count < 10)
+			    {
+				    s_log_count++;
+				    log_swferror(_("%s -- missing glyph for char %d. "
+						" Make sure character shapes for font %s are being exported "
+						"into your SWF file"),
+						__PRETTY_FUNCTION__,
+						code,
+						_font->get_name()
+				    );
+			    }
 
-			// Drop through and use index == -1; this will display
-			// using the empty-box glyph
-		}
+			    // Drop through and use index == -1; this will display
+			    // using the empty-box glyph
+		    }
+		);
 		text_glyph_record::glyph_entry	ge;
 		ge.m_glyph_index = index;
 		ge.m_glyph_advance = scale * _font->get_advance(index);
@@ -742,9 +1132,11 @@ edit_text_character::format_text()
 		rec.m_glyphs.push_back(ge);
 
 		x += ge.m_glyph_advance;
-
+		}
 		
-		if (x >= m_def->width() - m_def->get_right_margin() - WIDTH_FUDGE)
+after_x_advance:
+
+		if (x >= m_def->width() - m_def->get_right_margin() - PADDING_TWIPS)
 		{
 			// Whoops, we just exceeded the box width. 
 			// Do word-wrap if requested to do so.
@@ -752,9 +1144,10 @@ edit_text_character::format_text()
 			if ( ! m_def->do_word_wrap() )
 			{
 				static bool warned=false;
-				if ( ! warned ) {
-				log_warning("FIXME: implement no word wrap");
-				warned=true;
+				if ( ! warned )
+				{
+					log_unimpl("edit_text_character: no word wrap");
+					warned=true;
 				}
 			}
 
@@ -763,8 +1156,7 @@ edit_text_character::format_text()
 			// Close out this stretch of glyphs.
 			m_text_glyph_records.push_back(rec);
 			float	previous_x = x;
-
-			x = m_def->get_left_margin();
+			x = m_def->get_left_margin() + PADDING_TWIPS;
 			y += m_def->get_font_height() + leading;
 
 			// Start a new record on the next line.
@@ -828,7 +1220,6 @@ edit_text_character::format_text()
 
 	m_xcursor += static_cast<int>(extra_space);
 	m_ycursor -= m_def->get_font_height() + (_font->get_leading() - _font->get_descent()) * scale;
-
 }
 
 void
@@ -836,27 +1227,34 @@ edit_text_character::registerTextVariable()
 {
 //#define DEBUG_DYNTEXT_VARIABLES 1
 
-	if ( _text_variable_registered ) {
+	if ( _text_variable_registered )
+	{
 #ifdef DEBUG_DYNTEXT_VARIABLES
-	log_msg(" registerTextVariable() no-op call (alread registered)");
+		log_msg(_("registerTextVariable() no-op call (already registered)"));
 #endif
 		return;
 	}
 
-	const std::string& var_str = m_def->get_variable_name();
-
-	if ( var_str.empty() )
+	if ( _variable_name.empty() )
 	{
 #ifdef DEBUG_DYNTEXT_VARIABLES
-	log_msg(" string is empty, consider as registered");
+		log_msg(_("string is empty, consider as registered"));
 #endif
 		_text_variable_registered=true;
 		return;
 	}
 
+	std::string var_str = _variable_name;
+	VM& vm = VM::get();
+	if ( vm.getSWFVersion() < 7 )
+	{
+		boost::to_lower( var_str, vm.getLocale() );
+	}
+
 	const char* varname = var_str.c_str();
+
 #ifdef DEBUG_DYNTEXT_VARIABLES
-	log_msg(" VariableName: %s", var_str.c_str());
+	log_msg(_("VariableName: %s"), var_str.c_str());
 #endif
 
 	as_environment& env = get_environment();
@@ -867,11 +1265,11 @@ edit_text_character::registerTextVariable()
 	// If the variable string contains a path, we extract
 	// the appropriate target from it and update the variable
 	// name
-	tu_string path, var;
+	std::string path, var;
 	if ( as_environment::parse_path(varname, path, var) )
 	{
 #ifdef DEBUG_DYNTEXT_VARIABLES
-		log_msg("Variable text Path: %s, Var: %s", path.c_str(), var.c_str());
+		log_msg(_("Variable text Path: %s, Var: %s"), path.c_str(), var.c_str());
 #endif
 		// find target for the path component
 		// we use our parent's environment for this
@@ -884,7 +1282,7 @@ edit_text_character::registerTextVariable()
 	if ( ! target )
 	{
 		IF_VERBOSE_MALFORMED_SWF(
-			log_warning("VariableName associated to text field refer to an unknown target (%s). It is possible that the character will be instantiated later in the SWF stream. Gnash will try to register again on next access.", path.c_str());
+			log_swferror(_("VariableName associated to text field refer to an unknown target (%s). It is possible that the character will be instantiated later in the SWF stream. Gnash will try to register again on next access."), path.c_str());
 		);
 		return;
 	}
@@ -896,17 +1294,17 @@ edit_text_character::registerTextVariable()
 	// check if the VariableName already has a value,
 	// in that case update text value
 	as_value val;
-	if ( sprite->get_member(tu_stringi(varname), &val) )
+	if ( sprite->get_member(varname, &val) )
 	{
 #ifdef DEBUG_DYNTEXT_VARIABLES
-log_msg("target sprite (%p) does have a member named %s", (void*)sprite, varname);
+		log_msg(_("target sprite (%p) does have a member named %s"), (void*)sprite, varname);
 #endif
-		set_text_value(val.to_string());
+		set_text_value(val.to_string().c_str());
 	}
 #ifdef DEBUG_DYNTEXT_VARIABLES
 	else
 	{
-log_msg("target sprite (%p) does NOT have a member named %s (no problem, we'll add it)", (void*)sprite, varname);
+		log_msg(_("target sprite (%p) does NOT have a member named %s (no problem, we'll add it)"), (void*)sprite, varname);
 	}
 #endif
 
@@ -916,30 +1314,43 @@ log_msg("target sprite (%p) does NOT have a member named %s (no problem, we'll a
 	_text_variable_registered=true;
 }
 
-float
-edit_text_character::get_height() const
+void
+edit_text_character::set_variable_name(const std::string& newname)
 {
-	// @@ TODO should implement this in
-	// character and inherit into both here and sprite_instance
-	rect	transformed_rect;
-	transformed_rect.enclose_transformed_rect(get_world_matrix(), m_def->get_bounds());
-	return transformed_rect.height();
-
-	log_error("%s: not implmented yet", __PRETTY_FUNCTION__);
-	return 0;
+	if ( newname != _variable_name )
+	{
+		_variable_name = newname;
+		_text_variable_registered = false;
+	}
 }
 
-float
-edit_text_character::get_width() const
+void
+textfield_class_init(as_object& global)
 {
-	// @@ TODO should implement this in
-	// character and inherit into both here and sprite_instance
-	rect	transformed_rect;
-	transformed_rect.enclose_transformed_rect(get_world_matrix(), m_def->get_bounds());
-	return transformed_rect.width();
+	// This is going to be the global TextField "class"/"function"
+	static boost::intrusive_ptr<builtin_function> cl=NULL;
 
-	log_error("%s: not implmented yet", __PRETTY_FUNCTION__);
-	return 0;
+	if ( cl == NULL )
+	{
+		cl=new builtin_function(&textfield_ctor, getTextFieldInterface());
+		// replicate all interface to class, to be able to access
+		// all methods as static functions
+		attachTextFieldStaticMembers(*cl);
+		     
+	}
+
+	// Register _global.MovieClip
+	global.init_member("TextField", cl.get());
+}
+
+bool
+edit_text_character::pointInShape(float x, float y) const
+{
+	matrix wm = get_world_matrix();
+	point lp(x, y);
+	wm.transform_by_inverse(lp);
+	const rect& def_bounds = m_def->get_bounds();
+	return def_bounds.point_test(lp.m_x, lp.m_y);
 }
 
 } // namespace gnash
