@@ -1,10 +1,25 @@
-// font.cpp	-- Thatcher Ulrich <tu@tulrich.com> 2003
+// font.cpp:  ActionScript font handling, for Gnash.
+// 
+//   Copyright (C) 2006, 2007 Free Software Foundation, Inc.
+// 
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
+// 
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+//
 
-// This source code has been donated to the Public Domain.  Do
-// whatever you want with it.
-
-// A font type for gnash.
+/* $Id: font.cpp,v 1.32 2007/05/15 09:59:08 strk Exp $ */
 
+// Based on the public domain work of Thatcher Ulrich <tu@tulrich.com> 2003
 
 #include "font.h"
 #include "stream.h"
@@ -14,6 +29,7 @@
 #include "movie_definition.h"
 #include "shape_character_def.h"
 #include "swf.h"
+#include "GnashException.h"
 
 #include <utility> // for std::make_pair
 
@@ -47,7 +63,7 @@ namespace gnash {
 	{
 		if (index >= 0 && index < (int) m_glyphs.size())
 		{
-			return m_glyphs[index].get_ptr();
+			return m_glyphs[index].get();
 		}
 		else
 		{
@@ -112,8 +128,8 @@ namespace gnash {
 		}
 		else
 		{
-			assert (tag == SWF::DEFINEFONT2);
-			readDefineFont2(in, m);
+			assert (tag == SWF::DEFINEFONT2 || tag == SWF::DEFINEFONT3);
+			readDefineFont2_or_3(in, m);
 		}
 	}
 
@@ -121,15 +137,15 @@ namespace gnash {
 	void font::readDefineFont(stream* in, movie_definition* m)
 	{
 		IF_VERBOSE_PARSE (
-		log_parse("reading DefineFont");
+		log_parse(_("reading DefineFont"));
 		);
 
-		int	table_base = in->get_position();
+		unsigned long table_base = in->get_position();
 
 		// Read the glyph offsets.  Offsets
 		// are measured from the start of the
 		// offset table.
-		std::vector<int>	offsets;
+		std::vector<unsigned>	offsets;
 		offsets.push_back(in->read_u16());
 
 		IF_VERBOSE_PARSE (
@@ -155,23 +171,27 @@ namespace gnash {
 			{for (int i = 0; i < count; i++)
 			{
 				// Seek to the start of the shape data.
-				int	new_pos = table_base + offsets[i];
-				in->set_position(new_pos);
+				unsigned long new_pos = table_base + offsets[i];
+
+				if ( ! in->set_position(new_pos) )
+				{
+        				throw ParserException(_("Glyphs offset table corrupted in DefineFont tag"));
+				}
 
 				// Create & read the shape.
 				shape_character_def* s = new shape_character_def;
-				s->read(in, 2, false, m);
+				s->read(in, SWF::DEFINEFONT, false, m); 
 
 				m_glyphs[i] = s;
 			}}
 		}
 	}
 
-	// Read a DefineFont2 tag
-	void font::readDefineFont2(stream* in, movie_definition* m)
+	// Read a DefineFont2 or DefineFont3 tag
+	void font::readDefineFont2_or_3(stream* in, movie_definition* m)
 	{
 		IF_VERBOSE_PARSE (
-		log_parse("reading DefineFont2");
+		log_parse(_("reading DefineFont2 or DefineFont3"));
 		);
 
 		bool	has_layout = (in->read_uint(1) != 0);
@@ -184,14 +204,14 @@ namespace gnash {
 		m_is_bold = (in->read_uint(1) != 0);
 		uint8_t	reserved = in->read_u8();
 
-		// Inhibit warning.
+		// Inhibit compiler warning.
 		reserved = reserved;
 
 		m_name = in->read_string_with_length();
 
 		uint16_t glyph_count = in->read_u16();
 		
-		int	table_base = in->get_position();
+		unsigned long table_base = in->get_position();
 
 		// Read the glyph offsets.  Offsets
 		// are measured from the start of the
@@ -206,7 +226,7 @@ namespace gnash {
 				uint32_t off = in->read_u32();	
 
 				IF_VERBOSE_PARSE (
-				log_parse("Glyph %d at offset %u", i, off);
+				log_parse(_("Glyph %d at offset %u"), i, off);
 				);
 
 				offsets.push_back(off);
@@ -221,7 +241,7 @@ namespace gnash {
 				uint16_t off = in->read_u16();	
 
 				IF_VERBOSE_PARSE (
-				log_parse("Glyph %d at offset %u", i, off);
+				log_parse(_("Glyph %d at offset %u"), i, off);
 				);
 
 				offsets.push_back(off);
@@ -238,33 +258,38 @@ namespace gnash {
 			{for (int i = 0; i < glyph_count; i++)
 			{
 				// Seek to the start of the shape data.
-				int	new_pos = table_base + offsets[i];
+				unsigned long new_pos = table_base + offsets[i];
 
 				// It seems completely possible to
 				// have such seeks-back, see bug #16311
 				//assert(new_pos >= in->get_position());
 
-				in->set_position(new_pos);
+				if ( ! in->set_position(new_pos) )
+				{
+        				throw ParserException(_("Glyphs offset table corrupted in DefineFont2/3 tag"));
+				}
 
 				// Create & read the shape.
 				shape_character_def* s = new shape_character_def;
-				s->read(in, 22, false, m);
+				s->read(in, SWF::DEFINEFONT2, false, m); // .. or DEFINEFONT3 actually..
 
 				m_glyphs[i] = s;
 			}}
 
-			int	current_position = in->get_position();
+			unsigned long current_position = in->get_position();
 			if (font_code_offset + table_base != current_position)
 			{
 				// Bad offset!  Don't try to read any more.
-				log_warning("Bad offset in DefineFont2!");
+				IF_VERBOSE_MALFORMED_SWF(
+				log_swferror(_("Bad offset in DefineFont2"));
+				);
 				return;
 			}
 		}
 		else
 		{
 			// Skip the shape data.
-			int	new_pos = table_base + font_code_offset;
+			unsigned long new_pos = table_base + font_code_offset;
 			if (new_pos >= in->get_tag_end_position())
 			{
 				// No layout data!
@@ -324,7 +349,9 @@ namespace gnash {
 	// later using the character pair as the key.
 	if ( ! m_kerning_pairs.insert(std::make_pair(k, adjustment)).second )
 	{
-		log_warning("Repeated kerning pair found - ignoring\n");
+		IF_VERBOSE_MALFORMED_SWF(
+		log_swferror(_("Repeated kerning pair found - ignoring"));
+		);
 	}
 
 			}
@@ -343,9 +370,12 @@ namespace gnash {
 
 		if ( tag == SWF::DEFINEFONTINFO2 )
 		{
-			log_warning("DefineFontInfo2 partially implemented");
 			// See: SWFalexref/SWFalexref.html#tag_definefont2
-
+			static bool warned = false;
+			if ( ! warned ) {
+				log_unimpl(_("DefineFontInfo2 partially implemented"));
+				warned = true;
+			}
 		}
 
 		delete [] m_name;
@@ -374,7 +404,7 @@ namespace gnash {
 	// codes.
 	{
 		IF_VERBOSE_PARSE (
-		log_parse("reading code table at offset %d", in->get_position());
+		log_parse(_("reading code table at offset %lu"), in->get_position());
 		);
 
 		assert(m_code_table.empty());
@@ -408,14 +438,14 @@ namespace gnash {
 		{
 			int glyph_index = it->second;
 #if 0
-			log_msg("get_glyph_index(%u) returning %d",
+			log_msg(_("get_glyph_index(%u) returning %d"),
 				code, glyph_index);
 #endif
 			return glyph_index;
 		}
 
 #if 0
-		log_msg("get_glyph_index(%u) returning -1", code);
+		log_msg(_("get_glyph_index(%u) returning -1"), code);
 #endif
 		return -1;
 	}
@@ -435,7 +465,9 @@ namespace gnash {
 			if (s_logged == false)
 			{
 				s_logged = true;
-				log_error("empty advance table in font %s\n", get_name());
+				IF_VERBOSE_MALFORMED_SWF(
+				log_swferror(_("empty advance table in font %s"), get_name());
+				);
 			}
 			return 0;
 		}
@@ -483,7 +515,7 @@ namespace gnash {
 		out->write_le32(n);
 		for (int i = 0; i < n; i++)
 		{
-			shape_character_def*	s = m_glyphs[i].get_ptr();
+			shape_character_def*	s = m_glyphs[i].get();
 			if (s)
 			{
 				s->output_cached_data(out, options);
@@ -502,8 +534,8 @@ namespace gnash {
 		int	n = in->read_le32();
 		if (n != m_glyphs.size())
 		{
-			log_error("error reading cache file in font::input_cached_data() "
-				  "glyph count mismatch.\n");
+			log_error(_("error reading cache file in font::input_cached_data(): "
+				  "glyph count mismatch"));
 			in->go_to_end();	// ensure that no more data will be read from this stream.
 			return;
 		}
