@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,15 +10,11 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-/* $Id: gnash.h,v 1.68.2.2 2006/10/31 09:42:46 nihilus Exp $ */
-
 /// \mainpage
 ///
 ///  See Related Pages for movies and sprites informations
@@ -33,13 +29,12 @@
 
 #include "tu_config.h"
 
+#include <memory> // for auto_ptr
 #include <cctype>	// for poxy wchar_t
-#include <cstdarg>	// for va_list arg to movie_interface::call_method_args()
-#include <string>	// for movie_definition* create_movie(tu_file* in, const std::string& url);
+#include <cstdarg>	// for va_list arg to sprite_instance::call_method_args()
+#include <string>	// for movie_definition* create_movie(std::auto_ptr<tu_file> in, const std::string& url);
 
-#include "ref_counted.h" // for bitmap_info inheritance
-
-#include "as_value.h" // for register_component(...)
+#include "as_value.h" // FIXME: for as_c_function_ptr typedef (register_component)
 
 // FIXME: The local usage of these constants should probably be renamed in this
 // file because they conflict with definitions in the system header files. Luckily
@@ -58,20 +53,16 @@ namespace image { class image_base; class rgb; class rgba; }
 
 // forward decl
 namespace jpeg { class input; }
-class tu_string;
-class tu_stringi;
 
 namespace gnash {
 // Forward declarations.
 class action_buffer;
 class as_value;
-class bitmap_info;
-class character;
 class execute_tag;
 class font;
 class matrix;
 class movie;
-class movie_interface;
+class sprite_instance;
 class movie_definition;
 class render_handler;
 class resource;
@@ -79,6 +70,23 @@ class rgba;
 class sound_handler;
 class stream;
 class URL;
+class rect;
+
+// Sound callbacks stuff
+
+/// \brief
+/// Pass in a sound handler, so you can handle audio on behalf of
+/// gnash.  This is optional; if you don't set a handler, or set
+/// NULL, then sounds won't be played.
+///
+/// If you want sound support, you should set this at startup,
+/// before loading or playing any movies!
+///
+DSOEXPORT void	set_sound_handler(sound_handler* s);
+
+/// Get currently registered sound handler
+DSOEXPORT sound_handler* get_sound_handler();
+
 
 ///
 /// Log & error reporting control.
@@ -98,23 +106,12 @@ void	set_verbose_parse(bool verbose);
 /// want to display anything).
 DSOEXPORT void    set_render_handler(render_handler* s);
 
-/// Pass in a sound handler, so you can handle audio on behalf of
-/// gnash.  This is optional; if you don't set a handler, or set
-/// NULL, then sounds won't be played.
-///
-/// If you want sound support, you should set this at startup,
-/// before loading or playing any movies!
-DSOEXPORT void	set_sound_handler(sound_handler* s);
-
 /// Set the base url against which to resolve relative urls
 DSOEXPORT void set_base_url(const URL& url);
 
 /// Return base url
 DSOEXPORT const gnash::URL& get_base_url();
 
-/// You probably don't need this. (@@ make it private?)
-DSOEXPORT sound_handler* get_sound_handler();
-
 /// Signature of file opener callback function
 typedef tu_file* (*file_opener_callback)(const URL& url);
 
@@ -125,7 +122,7 @@ typedef void (*progress_callback)(unsigned int loaded_bytes, unsigned int total_
 void	register_progress_callback(progress_callback progress_handle);
 
 /// Signature of fscommand callback function
-typedef void (*fscommand_callback)(movie_interface* movie, const char* command, const char* arg);
+typedef void (*fscommand_callback)(sprite_instance* movie, const char* command, const char* arg);
 
 /// ActionScripts embedded in a movie can use the built-in
 /// fscommand() function to send data back to the host
@@ -133,13 +130,19 @@ typedef void (*fscommand_callback)(movie_interface* movie, const char* command,
 /// a handler, which will be called when the embedded scripts
 /// call fscommand().
 ///
-/// The handler gets the movie_interface* that the script is
+/// The handler gets the sprite_instance* that the script is
 /// embedded in, and the two string arguments passed by the
 /// script to fscommand().
 DSOEXPORT void	register_fscommand_callback(fscommand_callback handler);
 
 /// Use this to register gnash extension
-void register_component(const tu_stringi& name, as_c_function_ptr handler);
+//
+/// WARNING: does not convert name, make sure to pass a lowercase
+///          name if SWF version is < 7 ! It seems currently no code
+///          calls this function..
+///
+///
+void register_component(const std::string& name, as_c_function_ptr handler);
 
 /// Use this to control how finely curves are subdivided.  1.0
 /// is the default; it's a pretty good value.  Larger values
@@ -153,12 +156,7 @@ DSOEXPORT render_handler*	create_render_handler_xbox();
 DSOEXPORT render_handler*	create_render_handler_ogl();
 //DSOEXPORT render_handler*	create_render_handler_cairo(void* cairohandle);
 
-DSOEXPORT sound_handler*	create_sound_handler_sdl();
-DSOEXPORT sound_handler* create_sound_handler_gst();
-
 class font;
-class character_def;
-class sound_sample;
 
 /// For caching precomputed stuff.  Generally of
 /// interest to gnash_processor and programs like it.
@@ -175,26 +173,6 @@ public:
 };
 
 
-/// Try to grab movie info from the header of the given .swf file.
-//
-/// Sets *version to 0 if info can't be extracted.
-///
-/// You can pass NULL for any entries you're not interested in.
-/// In particular, using a NULL tag_count will avoid scanning
-/// the whole movie.
-///
-/// FIXME: use a stream here, so we can use an already opened one.
-///
-DSOEXPORT void	get_movie_info(
-	const URL&	url,
-	int*		version,
-	int*		width,
-	int*		height,
-	float*		frames_per_second,
-	int*		frame_count,
-	int*		tag_count
-	);
-
 /// Enable/disable attempts to read cache files (.gsc) when loading movies.
 DSOEXPORT void	set_use_cache_files(bool use_cache);
 	
@@ -213,7 +191,7 @@ DSOEXPORT void	set_use_cache_files(bool use_cache);
 ///
 /// This calls add_ref() on the newly created definition; call
 /// drop_ref() when you're done with it.
-/// Or use smart_ptr<T> from base/smart_ptr.h if you want.
+/// Or use boost::intrusive_ptr<T> from base/smart_ptr.h if you want.
 ///
 /// @@ Hm, need to think about these creation API's.  Perhaps
 /// divide it into "low level" and "high level" calls.  Also,
@@ -223,7 +201,23 @@ DSOEXPORT void	set_use_cache_files(bool use_cache);
 ///
 /// IFF real_url is given, the movie's url will be set to that value.
 ///
-movie_definition* create_movie(const URL& url, const char* real_url=NULL);
+/// @param url
+///	The URL to load the movie from.
+///
+/// @param real_url
+///	The url to encode as the _url member of the resulting
+///	movie definition. Use NULL if it is not different from
+///	the actual url (default). This is used to simulate a run from
+///	the official publication url.
+///
+/// @param startLoaderThread
+///	If false only the header will be read, and you'll need to call completeLoad
+///	on the returned movie_definition to actually start it. This is tipically 
+///	used to postpone parsing until a VirtualMachine is initialized.
+///	Initializing the VirtualMachine requires a target SWF version, which can
+///	be found in the SWF header.
+///
+movie_definition* create_movie(const URL& url, const char* real_url=NULL, bool startLoaderThread=true);
 
 /// Load a movie from an already opened stream.
 //
@@ -233,7 +227,23 @@ movie_definition* create_movie(const URL& url, const char* real_url=NULL);
 /// No attempt will be made to load associated .gsc (cache) files
 /// by this function.
 ///
-DSOEXPORT movie_definition* create_movie(tu_file* in, const std::string& url);
+/// @param in
+///	The stream to load the movie from. Ownership is transferred
+///	to the returned object.
+///
+/// @param url
+///	The url to use as the _url member of the resulting
+///	movie definition. This is required as it can not be
+///	derived from the tu_file.
+///
+/// @param startLoaderThread
+///	If false only the header will be read, and you'll need to call completeLoad
+///	on the returned movie_definition to actually start it. This is tipically 
+///	used to postpone parsing until a VirtualMachine is initialized.
+///	Initializing the VirtualMachine requires a target SWF version, which can
+///	be found in the SWF header.
+///
+DSOEXPORT movie_definition* create_movie(std::auto_ptr<tu_file> in, const std::string& url, bool startLoaderThread=true);
 
 /// Creates the movie from the given input stream. 
 //
@@ -263,11 +273,6 @@ enum create_font_shapes_flag
 	DO_NOT_LOAD_FONT_SHAPES
 };
 
-//movie_definition*	create_movie_no_recurse(
-//	tu_file*		input_stream,
-//	create_bitmaps_flag	cbf,
-//	create_font_shapes_flag cfs);
-
 /// \brief
 /// Create a gnash::movie_definition from the given URL
 //
@@ -287,11 +292,27 @@ enum create_font_shapes_flag
 ///
 /// This calls add_ref() on the newly created definition; call
 /// drop_ref() when you're done with it.
-/// Or use smart_ptr<T> from base/smart_ptr.h if you want.
+/// Or use boost::intrusive_ptr<T> from base/smart_ptr.h if you want.
 ///
 /// IFF real_url is given, the movie's url will be set to that value.
 ///
-DSOEXPORT movie_definition* create_library_movie(const URL& url, const char* real_url=NULL);
+/// @param url
+///	The URL to load the movie from.
+///
+/// @param real_url
+///	The url to encode as the _url member of the resulting
+///	movie definition. Use NULL if it is not different from
+///	the actual url (default). This is used to simulate a run from
+///	the official publication url.
+///
+/// @param startLoaderThread
+///	If false only the header will be read, and you'll need to call completeLoad
+///	on the returned movie_definition to actually start it. This is tipically 
+///	used to postpone parsing until a VirtualMachine is initialized.
+///	Initializing the VirtualMachine requires a target SWF version, which can
+///	be found in the SWF header.
+///
+DSOEXPORT movie_definition* create_library_movie(const URL& url, const char* real_url=NULL, bool startLoaderThread=true);
 	
 
 /// Helper to pregenerate cached data (basically, shape tesselations). 
@@ -377,106 +398,6 @@ void	draw_string(const font* f, float x, float y, float size, const char* text);
 }
 	
 	
-//
-// Sound callback handler.
-//
-	
-// You may define a subclass of this, and pass an instance to
-// set_sound_handler().
-class DSOEXPORT sound_handler
-{
-public:
-
-	struct sound_envelope
-	{
-		uint32_t m_mark44;
-		uint16_t m_level0;
-		uint16_t m_level1;
-	};
-
-	enum format_type
-	{
-		FORMAT_RAW = 0,		// unspecified format.  Useful for 8-bit sounds???
-		FORMAT_ADPCM = 1,	// gnash doesn't pass this through; it uncompresses and sends FORMAT_NATIVE16
-		FORMAT_MP3 = 2,
-		FORMAT_UNCOMPRESSED = 3,	// 16 bits/sample, little-endian
-		FORMAT_NELLYMOSER = 6,	// Mystery proprietary format; see nellymoser.com
-				
-		// gnash tries to convert data to this format when possible:
-		FORMAT_NATIVE16 = 7	// gnash extension: 16 bits/sample, native-endian
-	};
-	// If stereo is true, samples are interleaved w/ left sample first.
-	
-	// gnash calls at load-time with sound data, to be
-	// played later.  You should create a sample with the
-	// data, and return a handle that can be used to play
-	// it later.  If the data is in a format you can't
-	// deal with, then you can return 0 (for example), and
-	// then ignore 0's in play_sound() and delete_sound().
-	//
-	// Assign handles however you like.
-	virtual int	create_sound(
-		void*		data,
-		int		data_bytes,
-		int		sample_count,
-		format_type	format,
-		int		sample_rate,	/* one of 5512, 11025, 22050, 44100 */
-		bool		stereo
-		) = 0;
-
-	// gnash calls this to fill up soundstreams data
-	virtual long	fill_stream_data(void* data, int data_bytes, int sample_count, int handle_id) = 0;
-
-	//	Gives info about the format, samplerate and stereo of the sound in question;
-	virtual void get_info(int sound_handle, int* format, bool* stereo) = 0;
-
-	// gnash calls this when it wants you to play the defined sound.
-	// loop_count == 0 means play the sound once (1 means play it twice, etc)
-	virtual void	play_sound(int sound_handle, int loop_count, int secondOffset, long start, std::vector<sound_envelope>* envelopes) = 0;
-
-	//	stops all sounds currently playing in a SWF file without stopping the playhead.
-	//	Sounds set to stream will resume playing as the playhead moves over the frames they are in.
-	virtual void	stop_all_sounds() = 0;
-
-	//	returns the sound volume level as an integer from 0 to 100,
-	//	where 0 is off and 100 is full volume. The default setting is 100.
-	virtual int	get_volume(int sound_handle) = 0;
-	
-	//	A number from 0 to 100 representing a volume level. 
-	//	100 is full volume and 0 is no volume. The default setting is 100.
-	virtual void	set_volume(int sound_handle, int volume) = 0;
-		
-	// Stop the specified sound if it's playing.
-	// (Normally a full-featured sound API would take a
-	// handle specifying the *instance* of a playing
-	// sample, but SWF is not expressive that way.)
-	virtual void	stop_sound(int sound_handle) = 0;
-		
-	// gnash calls this when it's done with a particular sound.
-	virtual void	delete_sound(int sound_handle) = 0;
-		
-	// gnash calls this to mute audio
-	virtual void	mute() = 0;
-
-	// gnash calls this to unmute audio
-	virtual void	unmute() = 0;
-
-	//// @return Whether or not sound is muted.
-	virtual bool	is_muted() = 0;
-
-
-	virtual ~sound_handler() {};
-
-	// Utility function to uncompress ADPCM.
-	static void adpcm_expand(
-		void* data_out,
-		stream* in,
-		int sample_count,	// in stereo, this is number of *pairs* of samples
-		bool stereo);
-
-
-};
-	
 
 //
 // point: used by rect which is used by render_handler (otherwise would be in internal gnash_types.h)
@@ -500,81 +421,20 @@ public:
 
 	bool operator==(const point& p) const { return m_x == p.m_x && m_y == p.m_y; }
 
+	/// Return the square of the distance between this and other point.
+	float squareDistance(const point& other) const;
+
+	/// Return the distance between this and other point.
+	float distance(const point& other) const;
+
 	bool	bitwise_equal(const point& p) const;
 };
 
 
-/// Color transform type, used by render handler
-class DSOEXPORT cxform
-{
-public:
-    /// [RGBA][multiply, add]
-    float	m_[4][2];
-    
-    /// Initialize to the identity color transform (no transform)
-    cxform();
-    
-    /// Concatenate c's transform onto ours. 
-    //
-    /// When transforming colors, c's transform is applied
-    /// first, then ours.
-    ///
-    void concatenate(const cxform& c);
-    
-    /// Apply our transform to the given color; return the result.
-    rgba transform(const rgba in) const;
-    
-    /// Faster transform() method for loops (avoids creation of rgba object)
-    void transform(uint8_t& r, uint8_t& g, uint8_t& b, uint8_t& a) const;    
-    
-    /// Read RGB from the SWF input stream.
-    void read_rgb(stream* in);
-    
-    /// Read RGBA from the SWF input stream.
-    void read_rgba(stream* in);
-    
-    /// Force component values to be in range.
-    void clamp();
-    
-    /// Debug log.
-    void print() const;
-    
-    /// Returns true when the cxform equals identity (no transform)
-    bool is_identity() const;
-    
-    /// The identity color transform (no transform)
-    static cxform	identity;
-};
-
-
 //
 // texture and render callback handler.
 //
 
-/// Your render_handler creates bitmap_info's for gnash.  You
-/// need to subclass bitmap_info in order to add the
-/// information and functionality your app needs to render
-/// using textures.
-class DSOEXPORT bitmap_info : public ref_counted
-{
-public:
-	virtual void layout_image(image::image_base* /*im*/) { };
-	image::image_base*  m_suspended_image;
-
-	unsigned int	m_texture_id;		// nuke?
-	int		m_original_width;	// nuke?
-	int		m_original_height;	// nuke?
-		
-	bitmap_info()
-		:
-		m_suspended_image(NULL),
-		m_texture_id(0),
-		m_original_width(0),
-		m_original_height(0)
-		{
-		}
-};
-	
 /// Keyboard handling
 namespace key {
 enum code
@@ -681,14 +541,15 @@ enum code
 
 	KEYCOUNT
 };
+enum modifier
+{
+    MOD_NONE = 0,
+    MOD_SHIFT = 1,
+    MOD_CONTROL = 2,
+    MOD_ALT = 4
+};
 }	// end namespace key
 
-/// Key events are global throughout gnash.
-/// @@ Maybe someday make these local to the movie_interface?
-/// See \ref events_handling for more info
-DSOEXPORT void	notify_key_event(key::code k, bool down);
-
-
 /// Some optional helpers.
 namespace tools
 {
