@@ -1,5 +1,6 @@
+// impl.cpp:  Implement ActionScript tags, movie loading, library, for Gnash.
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,46 +11,31 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+//
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-// 
-//
 //
 
-/* $Id: impl.cpp,v 1.64.2.4 2006/11/13 17:10:44 strk Exp $ */
+/* $Id: impl.cpp,v 1.108 2007/06/01 11:02:18 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-// This needs to be included first for NetBSD systems or we get a weird
-// problem with pthread_t being defined too many times if we use any
-// STL containers.
-#ifdef HAVE_PTHREADS
-#include <pthread.h>
-#endif
-
-//#include <zlib.h>
-
 #include "tu_file.h"
 #include "utility.h"
 #include "action.h"
-//#include "button.h"
 #include "impl.h"
 #include "font.h"
 #include "fontlib.h"
 #include "log.h"
-//#include "morph2.h"
 #include "render.h"
 #include "shape.h"
-//#include "stream.h"
 #include "styles.h"
 #include "dlist.h"
 #include "timers.h"
 #include "image.h"
-//#include "jpeg.h"
 #include "zlib_adapter.h"
 #include "sprite_definition.h"
 #include "movie_def_impl.h"
@@ -60,6 +46,13 @@
 #include "URL.h"
 #include "StreamProvider.h"
 #include "sprite_instance.h"
+#include "VM.h"
+#include "BitmapMovieDefinition.h"
+#include "DefineFontAlignZonesTag.h"
+#include "PlaceObject2Tag.h"
+#include "RemoveObjectTag.h"
+#include "DoActionTag.h"
+#include "sound_handler.h" // for get_sound_handler
 
 #include <string>
 #include <map>
@@ -73,7 +66,7 @@ namespace gnash
 namespace globals { // gnash::globals
 
 	/// global StreamProvider
-	static StreamProvider streamProvider;
+	StreamProvider& streamProvider = StreamProvider::getDefaultInstance();
 
 	/// Base url (for relative urls resolution)
 	//
@@ -81,17 +74,22 @@ namespace globals { // gnash::globals
 	/// is an immutable one and needs to be set 
 	/// at construction time..
 	///
-	std::auto_ptr<URL> baseurl;
+	static std::auto_ptr<URL> baseurl;
 
 } // namespace gnash::global
 
+// global Sound handler stuff. Should this be moved to the VM class ?
+static sound_handler* _sound_handler = 0;
+void	set_sound_handler(sound_handler* s) { _sound_handler = s; }
+sound_handler*	get_sound_handler() { return _sound_handler; }
+
 void
 set_base_url(const URL& url)
 {
 	// can call this only once during a single run
 	assert(!globals::baseurl.get());
 	globals::baseurl.reset(new URL(url));
-	log_msg("Base url set to: %s", globals::baseurl->str().c_str());
+	log_msg(_("Base url set to: %s"), globals::baseurl->str().c_str());
 }
 
 const URL&
@@ -123,23 +121,23 @@ set_use_cache_files(bool use_cache)
     s_use_cache_files = use_cache;
 }
 
-// Keep a table of loader functions for the different tag types.
-SWF::TagLoadersTable s_tag_loaders;
-
 // Associate the specified tag type with the given tag loader
 // function.
 void
 register_tag_loader(SWF::tag_type t, SWF::TagLoadersTable::loader_function lf)
 {
-	bool loader_registered = s_tag_loaders.register_loader(t, lf);
-	//if ( !loader_registered )
-	// log_error("Duplicate loader registered for tag %d", t);
+	using SWF::TagLoadersTable;
+
+	TagLoadersTable& table = TagLoadersTable::getInstance();
+
+	bool loader_registered = table.register_loader(t, lf);
 	assert(loader_registered);
 }
 
 static void	ensure_loaders_registered()
 {
 	using namespace SWF::tag_loaders;
+	using namespace SWF;
 
 	static bool s_registered = false;
 	
@@ -151,15 +149,15 @@ static void	ensure_loaders_registered()
 	register_tag_loader(SWF::END,		end_loader);
 	register_tag_loader(SWF::DEFINESHAPE,	define_shape_loader);
 	register_tag_loader(SWF::FREECHARACTER, fixme_loader); // 03
-	register_tag_loader(SWF::PLACEOBJECT,	place_object_2_loader);
-	register_tag_loader(SWF::REMOVEOBJECT,	remove_object_2_loader);
+	register_tag_loader(SWF::PLACEOBJECT,	PlaceObject2Tag::loader);
+	register_tag_loader(SWF::REMOVEOBJECT,	RemoveObjectTag::loader); // 05
 	register_tag_loader(SWF::DEFINEBITS,	define_bits_jpeg_loader);
 	register_tag_loader(SWF::DEFINEBUTTON,	button_character_loader);
 	register_tag_loader(SWF::JPEGTABLES,	jpeg_tables_loader);
 	register_tag_loader(SWF::SETBACKGROUNDCOLOR, set_background_color_loader);
 	register_tag_loader(SWF::DEFINEFONT,	define_font_loader);
 	register_tag_loader(SWF::DEFINETEXT,	define_text_loader);
-	register_tag_loader(SWF::DOACTION,	do_action_loader);
+	register_tag_loader(SWF::DOACTION,	DoActionTag::doActionLoader);
 	register_tag_loader(SWF::DEFINEFONTINFO, define_font_info_loader);
 	register_tag_loader(SWF::DEFINEFONTINFO2, define_font_info_loader);
 	register_tag_loader(SWF::DEFINESOUND,	define_sound_loader);
@@ -173,11 +171,12 @@ static void	ensure_loaders_registered()
 	register_tag_loader(SWF::DEFINESHAPE2,	define_shape_loader);
 	register_tag_loader(SWF::DEFINEBUTTONCXFORM, fixme_loader); // 23
 	// "protect" tag; we're not an authoring tool so we don't care.
+	// (might be nice to dump the password instead..)
 	register_tag_loader(SWF::PROTECT,	null_loader);
 	register_tag_loader(SWF::PATHSAREPOSTSCRIPT, fixme_loader); // 25
-	register_tag_loader(SWF::PLACEOBJECT2,	place_object_2_loader);
+	register_tag_loader(SWF::PLACEOBJECT2,	PlaceObject2Tag::loader);
 	// 27 - _UNKNOWN_ unimplemented
-	register_tag_loader(SWF::REMOVEOBJECT2, remove_object_2_loader);
+	register_tag_loader(SWF::REMOVEOBJECT2, RemoveObjectTag::loader); // 28
 	register_tag_loader(SWF::SYNCFRAME,	fixme_loader); // 29
 	// 30 - _UNKNOWN_ unimplemented
 	register_tag_loader(SWF::FREEALL,	fixme_loader); // 31
@@ -190,14 +189,14 @@ static void	ensure_loaders_registered()
 	register_tag_loader(SWF::DEFINEVIDEO,	fixme_loader); // 38
 	register_tag_loader(SWF::DEFINESPRITE,  sprite_loader);
 	register_tag_loader(SWF::NAMECHARACTER, fixme_loader); // 40
-	register_tag_loader(SWF::SERIALNUMBER,  fixme_loader); // 41
+	register_tag_loader(SWF::SERIALNUMBER,  serialnumber_loader); // 41
 	register_tag_loader(SWF::DEFINETEXTFORMAT, fixme_loader); // 42
 	register_tag_loader(SWF::FRAMELABEL,	frame_label_loader); // 43
 	// 44 - _UNKNOWN_ unimplemented
 	register_tag_loader(SWF::SOUNDSTREAMHEAD2, sound_stream_head_loader); // 45
 	register_tag_loader(SWF::DEFINEMORPHSHAPE, define_shape_morph_loader);
 	register_tag_loader(SWF::FRAMETAG,	fixme_loader); // 47
-	register_tag_loader(SWF::DEFINEFONT2,	define_font_loader);
+	register_tag_loader(SWF::DEFINEFONT2,	define_font_loader); // 48
 	register_tag_loader(SWF::GENCOMMAND,	fixme_loader); // 49
 	register_tag_loader(SWF::DEFINECOMMANDOBJ, fixme_loader); // 50
 	register_tag_loader(SWF::CHARACTERSET,  fixme_loader); // 51
@@ -205,30 +204,40 @@ static void	ensure_loaders_registered()
 	// 53 - _UNKNOWN_ unimplemented
 	// 54 - _UNKNOWN_ unimplemented
 	// 55 - _UNKNOWN_ unimplemented
-	register_tag_loader(SWF::EXPORTASSETS,	export_loader);
-	register_tag_loader(SWF::IMPORTASSETS,  import_loader);
-	// 58 - _UNKNOWN_ unimplemented
-	register_tag_loader(SWF::INITACTION, do_init_action_loader);   
-	register_tag_loader(SWF::DEFINEVIDEOSTREAM, fixme_loader); // 60
+	register_tag_loader(SWF::EXPORTASSETS,	export_loader); // 56
+	register_tag_loader(SWF::IMPORTASSETS,  import_loader); // 57
 
-	register_tag_loader(SWF::VIDEOFRAME, fixme_loader); // 61
+	//  We're not an authoring tool so we don't care.
+	// (might be nice to dump the password instead..)
+	register_tag_loader(SWF::ENABLEDEBUGGER, null_loader);    // 58
 
-	register_tag_loader(SWF::FILEATTRIBUTES, fixme_loader); // 69
+	register_tag_loader(SWF::INITACTION, DoActionTag::doInitActionLoader);  // 59  
+
+	register_tag_loader(SWF::DEFINEVIDEOSTREAM, define_video_loader); // 60
+	register_tag_loader(SWF::VIDEOFRAME, video_loader); // 61
+	// 62 - _UNKNOWN_ unimplemented
+	// 63 - _UNKNOWN_ unimplemented
+	//  We're not an authoring tool so we don't care.
+	// (might be nice to dump the password instead..)
+	register_tag_loader(SWF::ENABLEDEBUGGER2, null_loader);    // 64
+	
+	register_tag_loader(SWF::FILEATTRIBUTES, file_attributes_loader); // 69
 	register_tag_loader(SWF::PLACEOBJECT3, fixme_loader); // 70
-	register_tag_loader(SWF::IMPORT2, fixme_loader); // 71
-	register_tag_loader(SWF::DEFINEALIGNZONES, fixme_loader); // 73
+	register_tag_loader(SWF::IMPORTASSETS2, import_loader); // 71
+
+	register_tag_loader(SWF::DEFINEALIGNZONES, DefineFontAlignZonesTag::loader); // 73
+
 	register_tag_loader(SWF::CSMTEXTSETTINGS, fixme_loader); // 74
-	register_tag_loader(SWF::DEFINEFONT3, fixme_loader); // 75
-	register_tag_loader(SWF::METADATA, fixme_loader); // 77
+	register_tag_loader(SWF::DEFINEFONT3, define_font_loader); // 75
+	register_tag_loader(SWF::METADATA, metadata_loader); // 77
 	register_tag_loader(SWF::DEFINESCALINGGRID, fixme_loader); // 78
 	register_tag_loader(SWF::DEFINESHAPE4, fixme_loader); // 83
 	register_tag_loader(SWF::DEFINEMORPHSHAPE2, fixme_loader); // 84
-
-
 }
 
 
 
+#if 0 // deprecated
 void	get_movie_info(
     const URL& url,
     int* version,
@@ -242,13 +251,13 @@ void	get_movie_info(
     // Put extracted info in the given vars.
     // Sets *version to 0 if info can't be extracted.
 {
-    //printf("%s: url is %s\n",  __PRETTY_FUNCTION__, url.str().c_str());
+    //log_msg(_("%s: url is %s"),  __PRETTY_FUNCTION__, url.str().c_str());
 
     tu_file*	in = globals::streamProvider.getStream(url);
     if (in == NULL || in->get_error() != TU_FILE_NO_ERROR) {
-	log_error("get_movie_info(): can't open '%s'\n", url.str().c_str());
+	log_error(_("get_movie_info(): can't open '%s'"), url.str().c_str());
 	if (version) *version = 0;
-	delete in;
+	//delete in;
 	return;
     }
     
@@ -261,19 +270,19 @@ void	get_movie_info(
     if ((header & 0x0FFFFFF) != 0x00535746
 	&& (header & 0x0FFFFFF) != 0x00535743) {
 	// ERROR
-	log_error("get_movie_info(): file '%s' does not start with a SWF header!\n", url.str().c_str());
+	log_error(_("get_movie_info(): file '%s' does not start with a SWF header"), url.str().c_str());
 	if (version) *version = 0;
-	delete in;
+	//delete in;
 	return;
     }
     bool	compressed = (header & 255) == 'C';
     
     tu_file*	original_in = NULL;
     if (compressed) {
-#if TU_CONFIG_LINK_TO_ZLIB == 0
-	log_error("get_movie_info(): can't read zipped SWF data; TU_CONFIG_LINK_TO_ZLIB is 0!\n");
+#ifndef HAVE_ZLIB_H
+	log_error(_("get_movie_info(): can't read zipped SWF data; gnash was compiled without zlib support"));
 	return;
-#endif
+#else
 	original_in = in;
 	
 	// Uncompress the input as we read it.
@@ -283,6 +292,7 @@ void	get_movie_info(
 	// it's not included in the compressed
 	// stream length.
 	file_length -= 8;
+#endif
     }
     
     stream	str(in);
@@ -312,54 +322,58 @@ void	get_movie_info(
 	    *tag_count = local_tag_count;
 	}
 
-    delete in;
-    delete original_in;
+    //delete in;
+    //delete original_in;
 }
+#endif
 
 // Create a movie_definition from a jpeg stream
 // NOTE: this method assumes this *is* a jpeg stream
 static movie_definition*
-create_jpeg_movie(tu_file* in, const std::string& /*url*/)
+create_jpeg_movie(std::auto_ptr<tu_file> in, const std::string& url)
 {
 	// FIXME: temporarly disabled
-	log_msg("Loading of jpegs unsupported");
-	return NULL;
+	//log_unimpl(_("Loading of jpegs"));
+	//return NULL;
 
 
-	bitmap_info* bi = NULL;
-	image::rgb* im = image::read_jpeg(in);
-	if (im != NULL) {
-		bi = render::create_bitmap_info_rgb(im);
-		delete im;
-	} else {
-		log_error("Can't read jpeg\n");
+	std::auto_ptr<image::rgb> im ( image::read_jpeg(in.get()) );
+
+	if ( ! im.get() )
+	{
+		log_error(_("Can't read jpeg"));
 		return NULL;
-	}
+	} 
+
+	BitmapMovieDefinition* mdef = new BitmapMovieDefinition(im, url);
+	//log_msg(_("BitmapMovieDefinition %p created"), mdef);
+	return mdef;
+
 
 	// FIXME: create a movie_definition from a jpeg
 	//bitmap_character*	 ch = new bitmap_character(bi);
-
 }
 
 // Get type of file looking at first bytes
-// return 'jpeg', 'swf' or 'unknown'
+// return "jpeg", "swf" or "unknown"
 //
 static std::string
 get_file_type(tu_file* in)
 {
 	in->set_position(0);
 
-	unsigned char buf[5];
-	memset(buf, 0, 5);
-	if ( 4 < in->read_bytes(buf, 4) )
+	unsigned char buf[3];
+	memset(buf, 0, 3);
+	if ( 3 < in->read_bytes(buf, 3) )
 	{
-		log_error("Can't read file header!\n");
+		log_error(_("Can't read file header"));
 		return "unknown";
 	}
 	
 	// This is the magic number for any JPEG format file
 	if ((buf[0] == 0xff) && (buf[1] == 0xd8) && (buf[2] == 0xff))
 	{
+		in->set_position(0);
 		return "jpeg";
 	}
 
@@ -368,87 +382,103 @@ get_file_type(tu_file* in)
 		(buf[1] == 'W') &&
 		(buf[2] == 'S') )
 	{
+		in->set_position(0);
 		return "swf";
 	}
+	
+	// Check if it is an swf embedded in a player (.exe-file)
+	if ((buf[0] == 'M') && (buf[1] == 'Z')) {
 
+		if ( 3 < in->read_bytes(buf, 3) ) return "unknown";
+
+		while (buf[0]!='F' && buf[0]!='C' || buf[1]!='W' || buf[2]!='S') {
+			buf[0] = buf[1];
+			buf[1] = buf[2];
+			buf[2] = in->read_byte();
+			if (in->get_eof()) return "unknown";
+		}
+		in->set_position(in->get_position()-3);
+		return "swf";
+	}
 	return "unknown";
 }
 
-// Create a movie_definition from an SWF stream
+// Create a movie_def_impl from an SWF stream
 // NOTE: this method assumes this *is* an SWF stream
-static movie_definition*
-create_swf_movie(tu_file* in, const std::string& url)
+//
+static movie_def_impl*
+create_swf_movie(std::auto_ptr<tu_file> in, const std::string& url, bool startLoaderThread)
 {
 
-	in->set_position(0);
+	// Avoid leaks on error 
+	std::auto_ptr<movie_def_impl> m(
+		new movie_def_impl(DO_LOAD_BITMAPS, DO_LOAD_FONT_SHAPES)
+		);
 
-	movie_def_impl* m = new movie_def_impl(DO_LOAD_BITMAPS,
-		DO_LOAD_FONT_SHAPES);
-	if ( ! m->read(in, url) ) return NULL;
+	if ( ! m->readHeader(in, url) )
+	{
+		return NULL;
+	}
 
-	return m;
+	if ( startLoaderThread && ! m->completeLoad() )
+	{
+		return NULL;
+	}
+
+	return m.release();
 }
 
 movie_definition*
-create_movie(tu_file* in, const std::string& url)
+create_movie(std::auto_ptr<tu_file> in, const std::string& url, bool startLoaderThread)
 {
-	assert(in);
+	assert(in.get());
 
 	ensure_loaders_registered();
 
 	// see if it's a jpeg or an swf
-	std::string type = get_file_type(in);
-
-	movie_definition* ret = NULL;
+	// TODO: use an integer code rather then a string !
+	std::string type = get_file_type(in.get());
 
 	if ( type == "jpeg" )
 	{
-		ret = create_jpeg_movie(in, url);
+		if ( startLoaderThread == false )
+		{
+			log_unimpl(_("Requested to keep from completely loading a movie, but the movie in question is a jpeg, for which we don't yet have the concept of a 'loading thread'"));
+		}
+		return create_jpeg_movie(in, url);
 	}
 	else if ( type == "swf" )
 	{
-		ret = create_swf_movie(in, url);
-	}
-	else
-	{
-		log_error("unknown file type\n");
-		ret = NULL;
+		return create_swf_movie(in, url, startLoaderThread);
 	}
 
-	if ( ! ret )
-	{
-		delete in;
-		return NULL;
-	}
-
-	//ret->add_ref(); // would leak forever
-
-	return ret;
+	log_error(_("unknown file type (%s)"), type.c_str());
+	return NULL;
 }
 
 movie_definition*
-create_movie(const URL& url, const char* reset_url)
+create_movie(const URL& url, const char* reset_url, bool startLoaderThread)
 {
 	// URL::str() returns by value, save it to a local string
 	std::string url_str = url.str();
 	const char* c_url = url_str.c_str();
 
-//	printf("%s: url is %s\n",  __PRETTY_FUNCTION__, c_url);
+//	log_msg(_("%s: url is %s"),  __PRETTY_FUNCTION__, c_url);
 
-	tu_file* in = globals::streamProvider.getStream(url);
-	if (in == NULL)
+	std::auto_ptr<tu_file> in ( globals::streamProvider.getStream(url) );
+	if ( ! in.get() )
 	{
-	    log_error("failed to open '%s'; can't create movie.\n", c_url);
+	    log_error(_("failed to open '%s'; can't create movie"), c_url);
 	    return NULL;
 	}
-	else if (in->get_error())
+	else if ( in->get_error() )
 	{
-	    log_error("streamProvider opener can't open '%s'\n", c_url);
+	    log_error(_("streamProvider opener can't open '%s'"), c_url);
 	    return NULL;
 	}
 
 	const char* movie_url = reset_url ? reset_url : c_url;
-	movie_definition* ret = create_movie(in, movie_url);
+	movie_definition* ret = create_movie(in, movie_url, startLoaderThread);
 
 	if (s_use_cache_files)
 	{
@@ -456,26 +486,27 @@ create_movie(const URL& url, const char* reset_url)
 		// WILL NOT WORK FOR NETWORK URLS, would need an hash
 		std::string cache_filename(movie_url);
 		cache_filename += ".gsc";
-		tu_file* cache_in = new tu_file(cache_filename.c_str(), "rb");
-		if (cache_in == NULL
+		std::auto_ptr<tu_file> cache_in ( new tu_file(cache_filename.c_str(), "rb") );
+		if (cache_in.get() == NULL
 			|| cache_in->get_error() != TU_FILE_NO_ERROR)
 		{
 			// Can't open cache file; don't sweat it.
-		        log_parse("note: couldn't open cache file '%s'\n",
+			IF_VERBOSE_PARSE (
+		        log_parse(_("note: couldn't open cache file '%s'"),
 					cache_filename.c_str());
+			)
 
 			// can't read cache, so generate font texture data.
 			ret->generate_font_bitmaps();
 		}
 		else
 		{
-			log_msg("Loading cache file %s",
+			log_msg(_("Loading cache file %s"),
 				cache_filename.c_str());
 			// Load the cached data.
-			ret->input_cached_data(cache_in);
+			ret->input_cached_data(cache_in.get());
 		}
 
-		delete cache_in;
 	}
 
 	return ret;
@@ -523,9 +554,30 @@ movie_definition*	create_movie_no_recurse(
 void	clear()
     // Maximum release of resources.
 {
+    // Ideally, we should make sure that function properly signals all threads
+    // about exiting and giving them a chance to cleanly exit.
+    //
+    // If we clear shared memory here we're going to leave threads possibly
+    // accessing deleted memory, which would trigger a segfault.
+    //
+    // My experience is that calling exit(), altought it has the same problem,
+    // reduces the chances of segfaulting ...
+    //
+    // We want this fixed anyway as exit()
+    // itselt can also trigger segfaults.
+    //
+    // See task task #6959 and depending items
+    //
+    std::cerr << "Any segfault past this message is likely due to improper threads cleanup." << std::endl;
+    exit(EXIT_SUCCESS);
+
     clear_library();
     fontlib::clear();
-    action_clear();
+
+    // By setting the soundhandler to NULL we avoid it being used
+    // after it's been de-referenced
+    set_sound_handler(NULL);
+
 }
 
 
@@ -534,7 +586,7 @@ void	clear()
 //
 
 
-//static stringi_hash< smart_ptr<movie_definition> >	s_movie_library;
+//static stringi_hash< boost::intrusive_ptr<movie_definition> >	s_movie_library;
 
 /// Library of SWF movies indexed by URL strings
 //
@@ -547,7 +599,7 @@ class MovieLibrary
 {
 private:
 
-	typedef std::map< std::string, smart_ptr<movie_definition> > container;
+	typedef std::map< std::string, boost::intrusive_ptr<movie_definition> > container;
 
 	container _map;
 
@@ -555,7 +607,7 @@ public:
 
 	MovieLibrary() {}
 
-	bool get(const std::string& key, smart_ptr<movie_definition>* ret)
+	bool get(const std::string& key, boost::intrusive_ptr<movie_definition>* ret)
 	{
 		container::iterator it = _map.find(key);
 		if ( it != _map.end() )
@@ -579,13 +631,12 @@ public:
 
 static MovieLibrary s_movie_library;
 
-static hash< movie_definition*, smart_ptr<movie_interface> >	s_movie_library_inst;
-static std::vector<movie_interface*> s_extern_sprites;
-static movie_interface* s_current_root;
+static hash< movie_definition*, boost::intrusive_ptr<sprite_instance> >	s_movie_library_inst;
+static std::vector<sprite_instance*> s_extern_sprites;
 
 static std::string s_workdir;
 
-void save_extern_movie(movie_interface* m)
+void save_extern_movie(sprite_instance* m)
 {
     s_extern_sprites.push_back(m);
 }
@@ -595,12 +646,12 @@ void delete_unused_root()
 {
     for (unsigned int i = 0; i < s_extern_sprites.size(); i++)
 	{
-	    movie_interface* root_m = s_extern_sprites[i];
+	    sprite_instance* root_m = s_extern_sprites[i];
 	    sprite_instance* m = root_m->get_root_movie();
       
 	    if (m->get_ref_count() < 2)
 		{
-		    log_action("extern movie deleted");
+		    log_action(_("extern movie deleted"));
 		    s_extern_sprites.erase(s_extern_sprites.begin() + i);
 		    i--;
 		    root_m->drop_ref();
@@ -609,16 +660,10 @@ void delete_unused_root()
 }
 //#endif // 0
 
-movie_interface* get_current_root()
+movie_root*
+get_current_root()
 {
-//    assert(s_current_root != NULL);
-    return s_current_root;
-}
-
-void set_current_root(movie_interface* m)
-{
-    assert(m != NULL);
-    s_current_root = m;
+	return &(VM::get().getRoot());
 }
 
 const char* get_workdir()
@@ -644,62 +689,81 @@ void	clear_library()
 // loaded it already.  Add it to our library on success, and
 // return a pointer to it.
 //
-movie_definition* create_library_movie(const URL& url, const char* real_url)
+movie_definition* create_library_movie(const URL& url, const char* real_url, bool startLoaderThread)
 {
-//    log_msg("%s: url is %s", __PRETTY_FUNCTION__, url.str().c_str());
+//    log_msg(_("%s: url is %s"), __PRETTY_FUNCTION__, url.str().c_str());
 
     // Use real_url as label for cache if available 
-    std::string cache_label = real_url ? real_url : url.str();
+    std::string cache_label = real_url ? URL(real_url).str() : url.str();
 
     // Is the movie already in the library?
     {
-	smart_ptr<movie_definition>	m;
+	boost::intrusive_ptr<movie_definition>	m;
 	if ( s_movie_library.get(cache_label, &m) )
 	    {
-    		log_msg(" movie already in library");
+    		log_msg(_("Movie %s already in library"), cache_label.c_str());
 		// Return cached movie.
 		// m->add_ref(); let caller add the ref, if needed
-		return m.get_ptr();
+		return m.get();
 	    }
     }
 
-    // Try to open a file under the filename.
-    movie_definition* mov = create_movie(url, real_url);
+	// Try to open a file under the filename, but DO NOT start
+	// the loader thread now to avoid IMPORT tag loaders from 
+	// calling create_library_movie() again and NOT finding
+	// the just-created movie.
+	movie_definition* mov = create_movie(url, real_url, false);
+	//log_msg(_("create_movie(%s, %s, false) returned %p"), url.str().c_str(), real_url, mov);
 
-    if (mov == NULL)
+	if (mov == NULL)
 	{
-	    log_error("couldn't load library movie '%s'\n", url.str().c_str());
-	    return NULL;
-	}
-    else
-	{
-	    s_movie_library.add(cache_label, mov);
+		log_error(_("Couldn't load library movie '%s'"),
+			url.str().c_str());
+		return NULL;
 	}
 
-    // mov->add_ref(); // let caller add the ref if needed
-    return mov;
+	// Movie is good, add to the library 
+	s_movie_library.add(cache_label, mov);
+    	log_msg(_("Movie %s (SWF%d) added to library"), cache_label.c_str(), mov->get_version());
+
+	// Now complete the load if the movie is an SWF movie
+	// 
+	// FIXME: add completeLoad() to movie_definition class
+	//        to allow loads of JPEG to use a loader thread
+	//        too...
+	//
+	if ( startLoaderThread )
+	{
+		movie_def_impl* mdi = dynamic_cast<movie_def_impl*>(mov);
+		if ( mdi ) {
+			mdi->completeLoad();
+		}
+	}
+
+	//log_msg(_("create_library_movie(%s, %s, startLoaderThread=%d) about to return %p"), url.str().c_str(), real_url, startLoaderThread, mov);
+	return mov;
 }
 
-movie_interface* create_library_movie_inst(movie_definition* md)
+sprite_instance* create_library_movie_inst(movie_definition* md)
 {
     // Is the movie instance already in the library?
     {
-	smart_ptr<movie_interface>	m;
+	boost::intrusive_ptr<sprite_instance>	m;
 	s_movie_library_inst.get(md, &m);
 	if (m != NULL)
 	    {
 		// Return cached movie instance.
 		// m->add_ref(); // let caller increment refcount
-		return m.get_ptr();
+		return m.get();
 	    }
     }
 
     // Try to create movie interface
-    movie_interface* mov = md->create_instance();
+    sprite_instance* mov = md->create_instance();
 
     if (mov == NULL)
 	{
-	    log_error("couldn't create instance\n");
+	    log_error(_("%s: couldn't create instance"), __FUNCTION__);
 
 	    return NULL;
 	}
@@ -748,10 +812,10 @@ void	precompute_cached_data(movie_definition* movie_def)
     } save_stuff_instance;
 
     // Need an instance.
-    gnash::movie_interface*	m = movie_def->create_instance();
+    gnash::sprite_instance*	m = movie_def->create_instance();
     if (m == NULL)
 	{
-	    log_error("precompute_cached_data can't create instance of movie\n");
+	    log_error(_("precompute_cached_data can't create instance of movie"));
 	    return;
 	}
 		
@@ -783,24 +847,24 @@ void	precompute_cached_data(movie_definition* movie_def)
 		    break;
 		}
 
-	    if (m->get_play_state() == gnash::movie_interface::STOP)
+	    if (m->get_play_state() == gnash::sprite_instance::STOP)
 		{
 		    // Kick the movie.
-		    //printf("kicking movie, kick ct = %d\n", kick_count);
+		    //log_msg(_("kicking movie, kick ct = %d"), kick_count);
 		    m->goto_frame(last_frame + 1);
-		    m->set_play_state(gnash::movie_interface::PLAY);
+		    m->set_play_state(gnash::sprite_instance::PLAY);
 		    kick_count++;
 
 		    if (kick_count > 10)
 			{
-			    //printf("movie is stalled; giving up on playing it through.\n");
+			    //log_msg(_("movie is stalled; giving up on playing it through"));
 			    break;
 			}
 		}
 	    else if (m->get_current_frame() < last_frame)
 		{
 		    // Hm, apparently we looped back.  Skip ahead...
-		    log_error("loop back; jumping to frame " SIZET_FMT "\n", last_frame);
+		    log_error(_("loop back; jumping to frame " SIZET_FMT), last_frame);
 		    m->goto_frame(last_frame + 1);
 		}
 	    else
