@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,6 +10,7 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -27,18 +28,25 @@
 
 #include <string>
 
-#include "impl.h"
+//#include "impl.h"
+#include "as_object.h" // for inheritance
+
 #include <sys/types.h>
-#ifndef HAVE_WINSOCK_H
+#if !defined(HAVE_WINSOCK_H) && !defined(__riscos__)
 # include <sys/ipc.h>
 # include <sys/shm.h>
-#else
+#elif !defined(__riscos__)
 # include <windows.h>
 # include <process.h>
 # include <fcntl.h>
 # include <io.h>
 #endif
 
+// Forward declarations
+namespace gnash {
+	class fn_call;
+}
+
 namespace gnash {
 
 #ifndef MAP_INHERIT
@@ -170,10 +178,10 @@ public:
 };
 
 //#ifdef ENABLE_TESTING 
-void shm_getname(const fn_call& fn);
-void shm_getsize(const fn_call& fn);
-void shm_getallocated(const fn_call& fn);
-void shm_exists(const fn_call& fn);
+as_value shm_getname(const fn_call& fn);
+as_value shm_getsize(const fn_call& fn);
+as_value shm_getallocated(const fn_call& fn);
+as_value shm_exists(const fn_call& fn);
 //#endif
 
 } // end of gnash namespace
