@@ -30,6 +30,10 @@ namespace gnash {
 	{
 	}
 
+	unsigned stream::read(char *buf, unsigned count)
+	{
+		return m_input->read_bytes(buf, count);
+	}
 
 // @@ better?
 // 	int	stream::read_uint(int bitcount)
@@ -50,13 +54,13 @@ namespace gnash {
 // 	}
 
 	
-	int	stream::read_uint(int bitcount)
+	unsigned stream::read_uint(unsigned short bitcount)
 	{
-		assert(bitcount <= 32 && bitcount >= 0);
+		assert(bitcount <= 32);
 			
 		uint32_t	value = 0;
 
-		int	bits_needed = bitcount;
+		unsigned short bits_needed = bitcount;
 		while (bits_needed > 0)
 		{
 			if (m_unused_bits) {
@@ -93,11 +97,11 @@ namespace gnash {
 	}
 
 
-	int	stream::read_sint(int bitcount)
+	int	stream::read_sint(unsigned short bitcount)
 	{
-		assert(bitcount <= 32 && bitcount >= 0);
+		assert(bitcount <= 32);
 
-		int32_t	value = (int32_t) read_uint(bitcount);
+		int32_t	value = int32_t(read_uint(bitcount));
 
 		// Sign extend...
 		if (value & (1 << (bitcount - 1))) {
@@ -183,6 +187,7 @@ namespace gnash {
 		align();
 
 		int	len = read_u8();
+		//log_msg("String length: %d", len);
 		if (len <= 0)
 		{
 			return NULL;
@@ -206,6 +211,13 @@ namespace gnash {
 		align();
 
 		unsigned int	len = read_u8();
+		read_string_with_length(len, to);
+	}
+
+	void stream::read_string_with_length(unsigned len, std::string& to)
+	{
+		align();
+
 		to.resize(len);
 
 		for (unsigned int i = 0; i < len; ++i)
@@ -216,31 +228,44 @@ namespace gnash {
 	}
 
 
-	int	stream::get_position()
+	unsigned long stream::get_position()
 	{
 		return m_input->get_position();
 	}
 
 
-	void	stream::set_position(int pos)
+	bool	stream::set_position(unsigned long pos)
 	{
 		align();
 
 		// If we're in a tag, make sure we're not seeking outside the tag.
 		if (m_tag_stack.size() > 0)
 		{
-			int	end_pos = m_tag_stack.back();
-			assert(pos <= end_pos);
-			end_pos = end_pos;	// inhibit warning
+			unsigned long end_pos = m_tag_stack.back();
+			if ( pos > end_pos )
+			{
+				log_error("Attempt to seek past the end of an opened tag");
+				// abort(); ?
+				return false;
+			}
 			// @@ check start pos somehow???
 		}
 
 		// Do the seek.
-		m_input->set_position(pos);
+		if ( m_input->set_position(pos) == TU_FILE_SEEK_ERROR )
+		{
+			// TODO: should we throw an exception ?
+			//       we might be called from an exception handler
+			//       so throwing here might be a double throw...
+			log_swferror(_("Unexpected end of stream"));
+			return false;
+		}
+
+		return true;
 	}
 
 
-	int	stream::get_tag_end_position()
+	unsigned long stream::get_tag_end_position()
 	{
 		assert(m_tag_stack.size() > 0);
 
@@ -251,6 +276,10 @@ namespace gnash {
 	SWF::tag_type stream::open_tag()
 	{
 		align();
+
+		// for debugging
+		unsigned long offset=get_position();
+
 		int	tag_header = read_u16();
 		int	tag_type = tag_header >> 6;
 		int	tag_length = tag_header & 0x3F;
@@ -258,16 +287,17 @@ namespace gnash {
 		if (tag_length == 0x3F) {
 			tag_length = m_input->read_le32();
 		}
-
-		IF_VERBOSE_PARSE (
-			log_parse("SWF[%u]: tag type = %d, tag length = %d",
-			get_position(), tag_type, tag_length);
-		);
+		_current_tag_length = tag_length;
 			
 		// Remember where the end of the tag is, so we can
 		// fast-forward past it when we're done reading it.
 		m_tag_stack.push_back(get_position() + tag_length);
 
+		IF_VERBOSE_PARSE (
+			log_parse("SWF[%lu]: tag type = %d, tag length = %d, end tag = %lu",
+			offset, tag_type, tag_length, m_tag_stack.back());
+		);
+
 		return static_cast<SWF::tag_type>(tag_type);
 	}
 
@@ -275,9 +305,17 @@ namespace gnash {
 	void	stream::close_tag()
 	{
 		assert(m_tag_stack.size() > 0);
-		int	end_pos = m_tag_stack.back();
+		unsigned long end_pos = m_tag_stack.back();
 		m_tag_stack.pop_back();
-		m_input->set_position(end_pos);
+
+		if ( m_input->set_position(end_pos) == TU_FILE_SEEK_ERROR )
+		{
+			log_error("Could not seek to end position");
+		}
+		else
+		{
+			assert(m_input->get_position() == end_pos);
+		}
 
 		m_unused_bits = 0;
 	}
