@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,6 +10,7 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -83,9 +84,11 @@ typedef enum
     FONTREF		  = 52,
     EXPORTASSETS          = 56,
     IMPORTASSETS          = 57,
+
+    /// See http://sswf.sourceforge.net/SWFalexref.html#tag_protectdebug
     ENABLEDEBUGGER	  = 58,
 
-    /// #initclip
+    /// For actions specified with initclip directive
     INITACTION		  = 59,
 
     DEFINEVIDEOSTREAM	  = 60,
@@ -94,6 +97,10 @@ typedef enum
     /// DefineFontInfo2 (swf6)
     DEFINEFONTINFO2       = 62,
 
+    /// See http://flasm.sourceforge.net/#protect
+    /// See http://sswf.sourceforge.net/SWFalexref.html#tag_protectdebug
+    ENABLEDEBUGGER2	  = 64,
+
     /// http://sswf.sourceforge.net/SWFalexref.html#tag_fileattributes
     FILEATTRIBUTES        = 69,
 
@@ -101,7 +108,7 @@ typedef enum
     PLACEOBJECT3          = 70,
 
     /// http://sswf.sourceforge.net/SWFalexref.html#tag_import2
-    IMPORT2               = 71,
+    IMPORTASSETS2         = 71,
 
     /// http://sswf.sourceforge.net/SWFalexref.html#tag_definefontalignzones
     DEFINEALIGNZONES      = 73,
@@ -124,6 +131,8 @@ typedef enum
     /// http://sswf.sourceforge.net/SWFalexref.html#tag_definemorphshape2
     DEFINEMORPHSHAPE2     = 84,
 
+    REFLEX                = 777,
+
     DEFINEBITSPTR         = 1023
 } tag_type;
 
@@ -219,11 +228,12 @@ typedef enum
     ACTION_CALLFRAME               = 0x9E,
     ACTION_GOTOEXPRESSION          = 0x9F,
 
-    /// action_buffer::doActionDelete
-    ACTION_DELETEVAR               = 0x3A,
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_delete
+    ACTION_DELETE                  = 0x3A,
 
-    /// action_buffer::doActionDelete2
-    ACTION_DELETE                  = 0x3B,
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_delete_all
+    /// The information in SWFalexref is pretty confusing, not sure it is correct...
+    ACTION_DELETE2                 = 0x3B,
 
     ACTION_VAREQUALS               = 0x3C, // DEFINELOCAL actually
     ACTION_CALLFUNCTION            = 0x3D,
@@ -265,8 +275,22 @@ typedef enum
     ACTION_INITOBJECT              = 0x43,
 
     ACTION_TYPEOF                  = 0x44,
+
+    /// SWF5
+    ///
+    /// Pop a value from the stack. If it is a valid movieclip push
+    /// it's target back on the stack (example: _level0.sprite1.sprite2).
+    ///
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_get_target
+    ///
     ACTION_TARGETPATH              = 0x45,
+
+    /// SWF5
+    ///
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_enumerate
+    ///
     ACTION_ENUMERATE               = 0x46,
+
     ACTION_NEWADD                  = 0x47,
     ACTION_NEWLESSTHAN             = 0x48,
 
@@ -276,13 +300,52 @@ typedef enum
     ///
     ACTION_NEWEQUALS               = 0x49,
 
+    /// SWF5
+    ///
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_number
+    ///
     ACTION_TONUMBER                = 0x4A,
+
+    /// SWF5
+    ///
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_string
+    ///
     ACTION_TOSTRING                = 0x4B,
+
+    /// SWF5
+    ///
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_duplicate
+    ///
     ACTION_DUP                     = 0x4C,
+
+    /// SWF5
+    ///
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_swap
+    ///
     ACTION_SWAP                    = 0x4D,
+
+    /// SWF5
+    ///
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_get_member
+    ///
     ACTION_GETMEMBER               = 0x4E,
+
+    /// SWF5
+    ///
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_set_member
+    ///
     ACTION_SETMEMBER               = 0x4F,
+
+    /// SWF5
+    ///
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_increment
+    ///
     ACTION_INCREMENT               = 0x50,
+
+    /// SWF5
+    ///
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_decrement
+    ///
     ACTION_DECREMENT               = 0x51,
 
     /// SWF5
@@ -386,6 +449,8 @@ typedef enum
     ///
     ACTION_EXTENDS                 = 0x69,
 
+    /// See:
+    /// http://sswf.sourceforge.net/SWFalexref.html#action_declare_dictionary
     ACTION_CONSTANTPOOL            = 0x88,
 
 
