@@ -2,12 +2,31 @@
 
 ## generate index.html
 {
-echo "<html><head><title>Test units</title></head><body>" 
-echo "<a href=embed.html>all tests in a single page</a>"
+
+ALLTESTS=$1
+shift
+
+echo "<html><head><title>Gnash - ActionScript tests</title></head><body>" 
+echo "<h1>Gnash - ActionScript tests</h1>"
+echo "<h2>All tests in a single SWF movie</h2>"
+echo "<p>"
+echo "<ul>"
+echo "<li><a href=${ALLTESTS}>${ALLTESTS}</a></li>"
+echo "</ul>"
+echo "</p>"
+echo "<p>"
+echo "This test is likely to overflow the 'visual traces' window."
+echo "If this is the case (you can't see the final report about number"
+echo "of succeeded / failed tests) consider running the single tests "
+echo "instead. We'll need to fix this."
+echo "</p>"
+echo "<h2>One SWF for each test</H2>"
+echo "<p>"
 echo "<ul>"
 for t in $@; do 
 	echo "<li><a href=$t>$t</a></li>" 
 done; 
+echo "</p>"
 echo "</body></html>" 
 } > index.html
 
@@ -35,18 +54,3 @@ done;
 echo "</body></html>" 
 } > embed.html
 
-## generate list for the wiki
-{
-for t in $@; do 
-	testname=`basename $t .swf`
-	{
-	echo "== $testname =="
-	echo 
-	echo "[http://www.gnu.org/software/gnash/testcases/v5/$t SWF5] "
-	echo "[http://www.gnu.org/software/gnash/testcases/v6/$t SWF6] "
-	echo "[http://www.gnu.org/software/gnash/testcases/v7/$t SWF7] "
-	echo "[http://www.gnu.org/software/gnash/testcases/v8/$t SWF8]"
-	echo 
-	} 
-done; 
-} > index.wiki
